#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using System.Collections.Generic;
using System.Windows.Forms;
using NinjaTrader.Gui.Design;
using System.Drawing.Design;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    
    [Description("Shows the volumes of Buys and Sells at each price level within the bars of the price chart.")]
    public class VolumeLadderMetroEditionV2 : GomDeltaIndicator
    {
       
		#region DrawData Class
		class DrawData
		{	
			public string FormattedData;
			public Font BAFont;
			public Font BABoldFont;
			public Font SideFont;
			public Font SideBoldFont;
			public Font BottomFont;
			public Font DiaFont;
			public Font DiaBoldFont;
			public Brush BAFontBrush;
			public Brush SideFontBrush;
			public Brush BottomFontBrush;
			public Color FillColor;
			public float FillStartRatio=0f;
			public float FillEndRatio=1f;
			public bool ifmax;  // used for checking volume to max volume in ladders.
			public Brush DiaStrongAskBrush;
			public Brush DiaStrongBidBrush;
			public Brush DiaWeakBrush;
			public Brush DiaNeutralBrush;
			public Brush CurrBrush;
			public Font CurrFont;
			public StringFormat CurrSF;
			public DrawData(Helper helper)
			{
				ifmax = false;
				FillColor=helper.colorBackGround;
				BAFontBrush = new SolidBrush(helper.baFontColor);
				SideFontBrush = new SolidBrush(helper.sideFontColor);
				BottomFontBrush = new SolidBrush(helper.bottomFontColor);
				
				DiaStrongAskBrush = new SolidBrush(helper.DiaStrongAskColor);
				DiaStrongBidBrush = new SolidBrush(helper.DiaStrongBidColor);
				DiaWeakBrush = new SolidBrush(helper.DiaWeakColor);
				DiaNeutralBrush = new SolidBrush(helper.DiaNeutralColor);
				
				BAFont = new Font(helper.baFont.Name,helper.baFont.Size,helper.baFont.Style);
				BABoldFont = new Font(helper.baFont.Name,helper.baFont.Size,FontStyle.Bold);
				SideFont = new Font(helper.sideFont.Name,helper.sideFont.Size,helper.sideFont.Style);
				SideBoldFont = new Font(helper.sideFont.Name,helper.sideFont.Size,FontStyle.Bold);
				BottomFont = new Font(helper.bottomFont.Name,helper.bottomFont.Size,helper.bottomFont.Style);
				DiaFont = new Font(helper.diaFont.Name,helper.diaFont.Size,helper.diaFont.Style);
				DiaBoldFont = new Font(helper.diaFont.Name,helper.diaFont.Size,FontStyle.Bold);
			}
		}
		#endregion
		
        #region ENUMS
		public enum LadderTypeEnum
		        {
					Bid,
					Ask,
					Delta,
					TotalVolume,
					DeltaDiff
				};
		
		public enum AccumulationTypeEnum {AllDays,NDays,ZeroDay}
		
		public enum FormattingType
				{
					Normal,
					Kilo,
					Kilo1Digit,
					Kilo2Digit,
					LotSize
				}
			
		public enum RatioType
				{
					MaximumOfLadder,
					FixedNumber
				}
		public enum RowNumber              
				{
					None =0,
					First =3,
					Second =23,
					Third =43,
					Fourth =63,
					Fifth =83,
					Sixth =103,
					Seventh =123,
					Eighth =143
				}
				
		public enum BottomViewType
				{
					None =0,
					Statistic =1,
					Graphic =2
				}
				
		public enum BottomGraphicType
				{
					Volume =0,
					DeltaVol =1,
					CumDeltaLine =2
				}
		public enum BottomGraphicHeight
				{
					Small = 70,
					Medium = 110,
					Large = 150,
					ExtraLarge = 210
				}
				
		
		public enum CellFillType
				{
					None = 0,
					Histogram = 1,
					Full = 2
				}
				
		public enum CenterLadderType   // Circulate : Multiply Key
				{
					BidAndAsk =0,
					VolAndDelta =1,
					BidXAsk =2,
					CustomLadder=3,
					DiagonalBidAsk =4,
					DiagonalDelta =5
				}
				
		public enum CellColorScheme    // Circulate : Ctrl + F4 
				{
					MaximumOfLadder =0,
					FixedNumber =1,
					DominateSide =2
				}
				
		public enum CustomLadderBase
				{
					Delta,
					Volume
				}
		public enum CustomLadderShape
				{
					Delta,
					Volume
				}
		public enum CustomLadderData
				{
					Delta,
					Volume,
					BidxAsk
				}
		public enum Direction
				{
					LeftToRight = 0,  // For Ask 
					RightToLeft = 1   // For Bid
				}
		
		public enum CellDataLocation
				{
					Right,
					Left,
					Center
				}
				
		public enum SideLadderWidth
				{
					Small = 40,
					Medium = 55,
					Large = 80
				}
				
		public enum LegendLocation
				{
					None,
					Left,
					Right
				}
				
		#endregion
				
		#region DrawBottomGraphic
	    static void DrawBottomGraphicView(Helper helper, int fbar, int lbar, Dictionary<int,BALadderClass> bal)
				{
					#region BaseRect-Init
					RectangleF baserect = new RectangleF(0f,0f,0f,0f);
					baserect.X = helper.bounds.Left + 2;
					baserect.Y = helper.bounds.Bottom - (int)helper.bgh;
					baserect.Width = helper.bounds.Width - 4;
					baserect.Height = (int)helper.bgh - 2;
					
					float textSpace = helper.bottomFont.GetHeight();
					float boxwidth = 2*(helper.BAcellwidth + helper.chartcontrol.BarWidth);
					float boxheight = (baserect.Height - textSpace - 6);
					float boxheight2 = (baserect.Height - 2*textSpace - 6);
					
					float xcl1 = helper.chartcontrol.GetXByBarIdx(helper.bars,helper.chartcontrol.LastBarPainted)+helper.chartcontrol.BarWidth+helper.BAcellwidth;
					float xcl = (xcl1 + (float)helper.bounds.Right)/2;
					float yc = baserect.Y + textSpace/2 +2;
					float yc2 = baserect.Y + textSpace +8;
					
					float boxtop = baserect.Y + textSpace + 6;
					float boxbottom = helper.bounds.Bottom - 2;
					
					helper.graphics.FillRectangle(new SolidBrush(Color.FromArgb(helper.bgBaseOpacity,helper.colorBottomGraphicBase)),baserect);
					helper.graphics.DrawLine(helper.myBGLinePen,baserect.X,baserect.Y,baserect.Width,baserect.Y);
					#endregion
					
					
					switch(helper.bottomgraphic){
						case BottomGraphicType.Volume:
							
							#region Volume
							int maxv = Int32.MinValue;
							for(int i = lbar; i >= fbar; i--)
							{
								int sum = bal[i].DeltaLadder.helper.dsVolume.Get(i);
								if(sum > maxv ) maxv = sum;
							}
							
							for(int i = lbar; i >= fbar; i--)
							{
								int curVol = bal[i].DeltaLadder.helper.dsVolume.Get(i);
								
								int embedBidVol = bal[i].BidLadder.Sum;
								int embedAskVol = bal[i].AskLadder.Sum;
								
								float xc = helper.chartcontrol.GetXByBarIdx(helper.bars,i);
								float bxc = xc - 9*boxwidth/40;
								float axc = xc + 9*boxwidth/40;
								
								RectangleF volRect = new RectangleF(0f,0f,0f,0f);
								volRect.X = xc - (9*boxwidth/20 + 3);
								volRect.Height = (maxv == 0) ? 0 : boxheight2*((float)curVol/(float)maxv);
								volRect.Width = 9*boxwidth/10 +6;
								volRect.Y = boxbottom - volRect.Height;
								
								RectangleF embedBidRect = new RectangleF(0f,0f,0f,0f);
								RectangleF embedAskRect = new RectangleF(0f,0f,0f,0f);
								
								embedBidRect.X = xc - (9*boxwidth/20 +2);
								embedBidRect.Height = (maxv == 0) ? 0 : boxheight2*((float)embedBidVol/(float)maxv);
								embedBidRect.Width = 9*boxwidth/20;
								embedBidRect.Y = boxbottom - embedBidRect.Height;
								
								embedAskRect.X = xc +2;
								embedAskRect.Height = (maxv == 0) ? 0 : boxheight2*((float)embedAskVol/(float)maxv);
								embedAskRect.Width = 9*boxwidth/20;
								embedAskRect.Y = boxbottom - embedAskRect.Height;
								
								decimal curbidp,curaskp;
								
								{
									curbidp = (curVol == 0) ? 0 : (decimal) (100*embedBidVol/(decimal)curVol);
									curaskp = (curVol == 0) ? 0 : (decimal) (100*embedAskVol/(decimal)curVol);
								}
								
								string vstr = LadderClass.Formatter(curVol,helper.ladderBAFormat,helper);
								helper.graphics.DrawString(vstr,helper.bottomFont,new SolidBrush(helper.bottomFontColor),xc,yc,centerSF);
								
								string bvstr = String.Format("{0:0.0}",curbidp);
								helper.graphics.DrawString(bvstr,helper.bottomFont,new SolidBrush(helper.bottomFontColor),bxc,yc2,centerSF);
								
								string avstr = String.Format("{0:0.0}",curaskp);
								helper.graphics.DrawString(avstr,helper.bottomFont,new SolidBrush(helper.bottomFontColor),axc,yc2,centerSF);
								
								helper.graphics.FillRectangle(new SolidBrush(helper.colorBottomGraphicVol),volRect);
								helper.graphics.DrawRectangle(helper.myBGLinePen,volRect.X,volRect.Y,volRect.Width,volRect.Height);
								
								helper.graphics.FillRectangle(new SolidBrush(helper.colorBottomGraphicBid),embedBidRect);
								helper.graphics.DrawRectangle(helper.myBGLinePen,embedBidRect.X,embedBidRect.Y,embedBidRect.Width,embedBidRect.Height);
								
								helper.graphics.FillRectangle(new SolidBrush(helper.colorBottomGraphicAsk),embedAskRect);
								helper.graphics.DrawRectangle(helper.myBGLinePen,embedAskRect.X,embedAskRect.Y,embedAskRect.Width,embedAskRect.Height);
								
							}
							
							helper.graphics.DrawString("VOLUME \r\nBid-Ask%",helper.bottomFont,new SolidBrush(helper.bottomFontColor),xcl,yc+textSpace/2,centerSF);
							break;
							#endregion
							
						case BottomGraphicType.DeltaVol:
							
							#region DeltaVolume
							int maxpsd = 0;               //max positive screen delta
							int minnsd = 0;               //min negative screen delta
							for(int i = lbar; i >= fbar; i--)
							{
								if(bal[i].MaxDelta > maxpsd ) maxpsd = bal[i].MaxDelta;
								if(bal[i].MinDelta < minnsd ) minnsd = bal[i].MinDelta;
							}
							float zeroy;
							float mx = Math.Abs(maxpsd);
							float mi = Math.Abs(minnsd);
							                                                           // Filtered Volume Fix.
							if(mx == 0 && mi == 0)zeroy = boxbottom - 0.5f*boxheight;  // changes made in zeroy calculation in respect of zero volume
							else zeroy = boxbottom - boxheight*(mi/(mi+mx));           // both on bid and ask in case of filtered volume ladder.
							
							helper.graphics.DrawLine(helper.myBGLinePen,baserect.X,zeroy,xcl1,zeroy);
							
							for(int i = lbar; i >= fbar; i--)
							{
								float xc = helper.chartcontrol.GetXByBarIdx(helper.bars,i);
								
								float mxc = (bal[i].MaxDelta == Int32.MinValue || bal[i].MaxDelta < 0) ? 0: Math.Abs(bal[i].MaxDelta);   //Filtered Volume Fix.(FVF)
								float mic = (bal[i].MinDelta == Int32.MaxValue || bal[i].MinDelta > 0) ? 0: Math.Abs(bal[i].MinDelta);   // Same.
								
								int curdv = bal[i].DeltaLadder.helper.dsDelta.Get(i);
								
								string curdvstr = LadderClass.Formatter(curdv,helper.ladderBAFormat,helper);
								helper.graphics.DrawString(curdvstr,helper.bottomFont,new SolidBrush(helper.bottomFontColor),xc,yc,centerSF);
								
								float maxLineY = (mx == 0) ? zeroy : zeroy - (zeroy - boxtop)*(mxc/mx);  // FVF
								helper.graphics.DrawLine(helper.myBGLinePen,xc,maxLineY,xc,zeroy);
								
								float minLineY = (mi == 0) ? zeroy : zeroy + (boxbottom - zeroy)*(mic/mi);   //FVF
								helper.graphics.DrawLine(helper.myBGLinePen,xc,zeroy,xc,minLineY);
								
								if(curdv > 0)
								{
									RectangleF deltaRect = new RectangleF(0f,0f,0f,0f);
									deltaRect.X = xc - (9*boxwidth/20 + 3);
									deltaRect.Y = zeroy - (zeroy - boxtop)*(curdv/mx);
									deltaRect.Width =  9*boxwidth/10 +6;
									deltaRect.Height = (zeroy - boxtop)*(curdv/mx);
									
									helper.graphics.FillRectangle(new SolidBrush(helper.colorBottomGraphicAsk),deltaRect);
									helper.graphics.DrawRectangle(helper.myBGLinePen,deltaRect.X,deltaRect.Y,deltaRect.Width,deltaRect.Height);
								}
								
								else if(curdv < 0 )
								{
									
									RectangleF deltaRect = new RectangleF(0f,0f,0f,0f);
									deltaRect.X = xc - (9*boxwidth/20 + 3);
									deltaRect.Y = zeroy; 
									deltaRect.Width =  9*boxwidth/10 +6;
									deltaRect.Height = (boxbottom - zeroy)*(Math.Abs(curdv)/mi);
									
									helper.graphics.FillRectangle(new SolidBrush(helper.colorBottomGraphicBid),deltaRect);
									helper.graphics.DrawRectangle(helper.myBGLinePen,deltaRect.X,deltaRect.Y,deltaRect.Width,deltaRect.Height);
								}
							}
							helper.graphics.DrawString("Delta\r\nVolume",helper.bottomFont,new SolidBrush(helper.bottomFontColor),xcl,yc+textSpace/2,centerSF);
							break;
							#endregion
							
						case BottomGraphicType.CumDeltaLine:
							
							#region CummulativeDeltaLine
							int maxCumD = Int32.MinValue;
							int minCumD = Int32.MaxValue;
							
							int cd;
							for(int i = lbar; i >= fbar; i--)
							{
								cd = helper.dsCumCD.Get(i);
								if(cd > maxCumD ) maxCumD = cd;
								if(cd < minCumD ) minCumD = cd;
							}
							
							float diff = (float) ( maxCumD - minCumD);
							if(maxCumD > 0 && minCumD < 0)
							{
								zeroy = boxbottom - boxheight*((float)Math.Abs(minCumD)/diff);
							    helper.graphics.DrawLine(helper.myBGLinePen,baserect.X,zeroy,xcl1,zeroy);
							}
							
							
							for(int i = lbar; i >= fbar; i--)
							{
								float cdy,lastcdy;
								cd = helper.dsCumCD.Get(i);
								if(diff == 0)
								{
									cdy = (cd > 0) ? boxbottom - boxheight : boxbottom;
								}
								else cdy = boxbottom - boxheight*((float)(cd - minCumD))/diff;
								float xc = helper.chartcontrol.GetXByBarIdx(helper.bars,i);
								
								string cdstr = LadderClass.Formatter(cd,helper.ladderBAFormat,helper);
								helper.graphics.DrawString(cdstr,helper.bottomFont,new SolidBrush(helper.bottomFontColor),xc,yc,centerSF);
								
								if(i >= 1 && i > fbar)
								{
									
									int lastcd = helper.dsCumCD.Get(i-1);
									if(diff == 0)
									{
										lastcdy = (lastcd > 0) ?  boxbottom - boxheight : boxbottom;
									}
									else lastcdy = boxbottom - boxheight*((float)(lastcd - minCumD))/diff;
									float lastxc = helper.chartcontrol.GetXByBarIdx(helper.bars,i-1);
									
									helper.graphics.DrawLine(helper.myBGLinePen,xc,cdy,lastxc,lastcdy);
								}
							    if(cd >=0 ) helper.graphics.FillEllipse(new SolidBrush(helper.colorBottomGraphicAsk),xc-3,cdy-3,6,6);
								else if (cd < 0 ) helper.graphics.FillEllipse(new SolidBrush(helper.colorBottomGraphicBid),xc-3,cdy-3,6,6);
							    helper.graphics.DrawEllipse(helper.myBGLinePen,xc-3,cdy-3,6,6);
								
							}
							
							helper.graphics.DrawString("CumDelta\r\nLine",helper.bottomFont,new SolidBrush(helper.bottomFontColor),xcl,yc+textSpace/2,centerSF);
							break;
							#endregion
						
					}
					
				}
		#endregion
		
		#region DrawMETHOD BACell/COTCell/SideCell/BottomBoxes/LegendBox
		static void DrawCOTAndRevCell(RectangleF rect,DrawData param, Helper helper)//Basic Draw for COTs and Reversals	
			{
				try{
					
				RectangleF rectToFill=new RectangleF();
				rectToFill.X=rect.X+param.FillStartRatio*rect.Width;
				rectToFill.Width=rect.Width*(param.FillEndRatio-param.FillStartRatio);
				rectToFill.Y=rect.Y;
				rectToFill.Height=rect.Height;

				helper.graphics.FillRectangle(new SolidBrush(param.FillColor),rectToFill);
				{	
					
					helper.graphics.DrawRectangle(myBlackPen,rect.X,rect.Y,rect.Width, rect.Height);
					helper.graphics.DrawString(param.FormattedData, param.BAFont,param.BAFontBrush,rect,centerSF);
					
				}
				
				} catch (Exception ex){
				return;
				}	
			}
			
			
			static void DrawBottomLegend(float x, float width, Helper helper, string legend, int rn)  // Draw for Legend
			{
				try{
					RectangleF legendRect = new RectangleF();
				
				    legendRect.X = x;
					legendRect.Height = 2f+helper.bottomFont.GetHeight();
					legendRect.Y = helper.bounds.Bottom-(float)rn-legendRect.Height;
					legendRect.Width = width;
					
					helper.graphics.DrawRectangle(legendPen,legendRect.X-1,legendRect.Y-1,legendRect.Width+2,legendRect.Height+2);
					helper.graphics.FillRectangle(new SolidBrush(helper.legendBackColor),legendRect);
					helper.graphics.DrawString(legend,helper.bottomFont,new SolidBrush(helper.bottomFontColor),legendRect.X,legendRect.Y);
					} catch (Exception ex){
				return;
				}
			}
			
			
			static void DrawBottomBox(RectangleF rect,DrawData param, Helper helper)//Basic Draw for  Bottom Boxes	
			{
				try{
					
				RectangleF rectToFill=new RectangleF();
				rectToFill.X=rect.X+param.FillStartRatio*rect.Width;
				rectToFill.Width=rect.Width*(param.FillEndRatio-param.FillStartRatio);
				rectToFill.Y=rect.Y;
				rectToFill.Height=rect.Height;

				helper.graphics.FillRectangle(new SolidBrush(param.FillColor),rectToFill);
				
				{	
					
					helper.graphics.DrawRectangle(myBlackPen,rect.X,rect.Y,rect.Width, rect.Height);
					helper.graphics.DrawString(param.FormattedData, param.BottomFont,param.BottomFontBrush,rect,centerSF);
					
				}
				
				} catch (Exception ex){
				return;
				}	
			}
			
			static void DrawDataBox(RectangleF rect, DrawData param, Helper helper, Font font)
			{
				rect.Y = rect.Y - (float)(.5*rect.Height);
				helper.graphics.DrawString(param.FormattedData, font,param.CurrBrush,rect,param.CurrSF);	
			}
			
			static void DrawFillerBox(RectangleF rect,DrawData param, Helper helper, Direction direction, bool showBorder) // direction = 0 for Ask, 1 for Bid
			{
				RectangleF rectToFill=new RectangleF();
				rectToFill.Width = rect.Width*param.FillEndRatio;
				rectToFill.X = rect.X + ((int)direction)*(1-param.FillEndRatio)*rect.Width;
				rectToFill.Y=rect.Y-(float)(.5*rect.Height);
				rectToFill.Height = rect.Height;
				
				helper.graphics.FillRectangle(new SolidBrush(param.FillColor),rectToFill);
				
				if(showBorder)
				helper.graphics.DrawRectangle(helper.myBAborderPen,rectToFill.X,rectToFill.Y,rectToFill.Width, rect.Height);
			}
			
		#endregion
			
		#region LADDER Class	
		class LadderClass
		{	
			//Fields & Properties
			private Dictionary<int,int> cells=new Dictionary<int,int>();
			public Dictionary<int,int> Cells {get {return cells;}}
								
			private int _minPriceLevel=Int32.MaxValue;
			public int MinPriceLevel{ get {return _minPriceLevel;}}

			private int _maxPriceLevel=Int32.MinValue;
			public int MaxPriceLevel{get {return _maxPriceLevel;}}
			
			private int _sum=0;
			public int Sum{get {return _sum;}}
						
			private int _maxLadderValue=Int32.MinValue;
			public int MaxLadderValue{get {return _maxLadderValue;} set { _maxLadderValue = value;}}
			
			private int _maxLadderValuePriceTick;
			public int MaxLadderValuePriceTick{get {return _maxLadderValuePriceTick;} set { _maxLadderValuePriceTick = value;}}
			
			private int _max2LadderValue=Int32.MinValue;
			public int Max2LadderValue{get {return _max2LadderValue;} set { _max2LadderValue = value;}}
			
			private int _max2LadderValuePriceTick;
			public int Max2LadderValuePriceTick{get {return _max2LadderValuePriceTick;} set { _max2LadderValuePriceTick = value;}}
			
			public delegate DrawData  FillCellsDelegate (int volume, int maxValue,int priceTick);
			protected FillCellsDelegate _fillCells;
			public FillCellsDelegate FillCells {set{_fillCells=value;}}
					
			public Helper _helper;
			public Helper helper { get {return _helper;}}	
					
			//Constructors
			public  LadderClass(Helper paramHelper)
			{
				_helper=paramHelper;
			}
						
			
			public void ComputeMax()
			{
				_maxLadderValue=Int32.MinValue;
				for(int curTick = _minPriceLevel; curTick <= _maxPriceLevel; curTick++)
				{
					if(Math.Abs(cells[curTick]) > _maxLadderValue )
					{
						_maxLadderValue = Math.Abs(Cells[curTick]);
						_maxLadderValuePriceTick = curTick;
					}
				}
			}
			
				
			public LadderClass(LadderClass previousLadder,Helper paramHelper):this(paramHelper)
			{
				if (previousLadder!=null)
				{
					foreach(KeyValuePair<int,int> iteratorpair in previousLadder.Cells)
						{  
							cells.Add(iteratorpair.Key,iteratorpair.Value);					
						}
					_minPriceLevel=previousLadder._minPriceLevel;
					_maxPriceLevel=previousLadder._maxPriceLevel;
				}
			}
			
			public void Add(LadderClass ladderToAdd)
			{
				if (ladderToAdd!=null)
				{
					foreach(KeyValuePair<int,int> iteratorpair in ladderToAdd.Cells)
						{  
							this.Increment(iteratorpair.Key,iteratorpair.Value,false);					
						}
				}
			}
			
			//Ladder management Methods
			public void CreatePriceLevel(int priceTick)
			{
				if(cells.Count == 0)
				{
					cells.Add(priceTick,0);
					
					_minPriceLevel=Math.Min(MinPriceLevel,priceTick);
				    _maxPriceLevel=Math.Max(MaxPriceLevel,priceTick);
				}
				else 
				{
					if(priceTick > _maxPriceLevel)
					{
					    for(int curTick = _maxPriceLevel + 1; curTick <= priceTick; curTick++)
						if(!cells.ContainsKey(curTick))cells.Add(curTick,0);
					}
					else if(priceTick < _minPriceLevel)
					{
						    for(int curTick = _minPriceLevel - 1; curTick >= priceTick; curTick--)
							if(!cells.ContainsKey(curTick))cells.Add(curTick,0);
					}
					
					_minPriceLevel=Math.Min(MinPriceLevel,priceTick);
				    _maxPriceLevel=Math.Max(MaxPriceLevel,priceTick);
					
				}
			}
			
			public void Increment(int priceTick, int dataToAdd, bool inLineCalcMax)
			{				
				if( !cells.ContainsKey(priceTick))
					CreatePriceLevel(priceTick);				

				cells[priceTick]+=dataToAdd;
				if (inLineCalcMax)
					_maxLadderValue= Math.Max(_maxLadderValue,Math.Abs(cells[priceTick]));	
				_sum += dataToAdd;	
			}                                                
	
			public void Set(int priceTick, int dataToAdd)   
			{				                                
				if( !cells.ContainsKey(priceTick))
					CreatePriceLevel(priceTick);				

				cells[priceTick]=dataToAdd;
			}
			
			public void CopyWithOffset(LadderClass ladderToAdd,int offset, bool ladderpostive)
			{
				if (ladderToAdd!=null)
				{
					foreach(KeyValuePair<int,int> iteratorpair in ladderToAdd.Cells)
						{  
							if( !cells.ContainsKey(iteratorpair.Key))
								CreatePriceLevel(iteratorpair.Key);				

							cells[iteratorpair.Key]=((ladderpostive)?1:-1)*iteratorpair.Value+offset;
						}
				}
			}
			
			
			//Graphical  Methods
			public void  DrawBorder(float x, float cellWidth,Pen pen) // Draws a border around whole Bid or Ask side ladder.
			{
				if (cells.Count>0)
				helper.graphics.DrawRectangle(pen,x,helper.GetY(MaxPriceLevel+0.5f),cellWidth,helper.scalingfactor*(MaxPriceLevel-MinPriceLevel+1));
			}
			
			public void  DrawCell(float x, float cellWidth,int curTick, Pen pen)//draws a rectangle at latest price level (current tick)
			{	
				
				if ((curTick<=helper.maxTick) && (curTick>=helper.minTick))
					helper.graphics.DrawRectangle(pen,x,helper.GetY((float)((float)curTick+.5f)),1f+cellWidth,helper.scalingfactor);// ouline for last tick
			}
			
			public void EmptyLadder(float x,float cellWidth)
			{	
				if (Cells.Count>0)
				{
					int minPrice=Math.Max(helper.minTick,MinPriceLevel);
					int maxPrice=Math.Min(helper.maxTick,MaxPriceLevel);
			
					helper.graphics.FillRectangle(helper.myWhiteBrush,x,helper.GetY(maxPrice+.5f),cellWidth,helper.scalingfactor*(maxPrice-minPrice+1));
				}
			}
			
			
			public void DrawGeneralLadderFillCells(float x,float cellWidth,Direction direction,bool showBorder,bool showFill,bool showData) // direction 0 = Ask , 1 = Bid
			{	
				RectangleF curLadderCellRect= new RectangleF(x,0,cellWidth,helper.scalingfactor);
				int curTick,curVol;
				DrawData curDrawData;
				
				curLadderCellRect.X=x;
				foreach (KeyValuePair<int, int> cell in Cells)
					{ 
						curTick=cell.Key;
						
						if ((curTick<=helper.maxTick) && (curTick>=helper.minTick))
						{
							curVol=cell.Value;
							curLadderCellRect.Y=helper.GetY(curTick);

							curDrawData=_fillCells(curVol,_maxLadderValue,curTick);
							if(showFill)DrawFillerBox(curLadderCellRect,curDrawData,helper,direction,showBorder);
							if(showData)DrawDataBox(curLadderCellRect,curDrawData,helper,curDrawData.CurrFont);
							curDrawData=null;
						}
					}
			}
			
			
			 //Volume To String Formatter
			public static string Formatter(int vol,FormattingType format,Helper helper)
			{
				
				if (format==FormattingType.Kilo)
					 return (((float)((vol)/1000f)).ToString("F1"));
				else if (format==FormattingType.Kilo1Digit)
					return (((float)((vol)/1000f)).ToString("F2"));
				else if (format==FormattingType.Kilo2Digit)
					return (((float)((vol)/1000f)).ToString("F3"));
				else if (format==FormattingType.LotSize)
				    return (((float)((vol)/helper.lotSize)).ToString("F0"));
				else
					return vol.ToString("N0");
				
			}
		}
		#endregion
		
		#region BA LADDER Class
		sealed class BALadderClass
		{  
			#region Field, Properties , Constructor
 			private LadderClass _bidLadder;
			public LadderClass BidLadder{get {return  _bidLadder;}}
			
			private LadderClass _askLadder;
			public LadderClass AskLadder{get {return  _askLadder;}}
			
			private LadderClass _cdLadder;
			public LadderClass CDLadder{get {return  _cdLadder;}}
			
			private LadderClass _volumeLadder;     //volume @price
			public LadderClass VolumeLadder{get {return  _volumeLadder;}}
			
			private LadderClass _deltaLadder;     //delta @price
			public LadderClass DeltaLadder{get {return  _deltaLadder;}}
			
			private LadderClass _diadeltaLadder;     //diadelta @price
			public LadderClass DiaDeltaLadder{get {return  _diadeltaLadder;}}
			
			private int _barIndex;
			private int _currentdelta=0;
			private int _cumdelta=0;
			public int _Volume=0;
			
			private bool _isFirstTickAtBid=false;		// Remember if bar opened at bid price (true) or at ask price (false).
			private bool _isLastTickAtBid=false;
			
			private Helper _helper;
			private Helper helper{get {return _helper;}}
			
			private int BAlastTickPrice;
			private int BAfirstTickPrice;			// open price of the bar.
			
			public int HighCOT=0,LowCOT=0;
			private int x=0;
			
			private int _deltamax=Int32.MinValue; // Calculates teh max delta attained per bar
			public int MaxDelta{get {return  _deltamax;}}
			
			private int _deltamin=Int32.MaxValue; // calulates the min delta attained per bar
			public int MinDelta{get {return  _deltamin;}}
			
			private int _opendelta=0;
			public int OpenDelta{get {return _opendelta;}}
			
			private int _start =0;
			public int Start{get {return _start;}}
			
			private int _finish=0;
			public int Finish{get {return _finish;}}
			
			private int _ticknumber =0;
			public int TickNumber{get {return _ticknumber;}}
			
			private int _maxdeltaticknumber =0;
			public int MaxDTN{get {return _maxdeltaticknumber;}}
			
			private int _mindeltaticknumber =0;
			public int MinDTN{get {return _mindeltaticknumber;}}
			
			
			//Constructors 
			public BALadderClass(int index,Helper paramHelper) 
			{ 			
				_helper=paramHelper;
				
				_bidLadder=new LadderClass(paramHelper);
				_askLadder=new LadderClass(paramHelper);				
				_cdLadder=new LadderClass(paramHelper);
				_volumeLadder=new LadderClass(paramHelper);
				_deltaLadder=new LadderClass(paramHelper);
				_diadeltaLadder=new LadderClass(paramHelper);
				
				_barIndex=index; 
							
			}
			#endregion

			//Ladder Management methods
			#region Add Ask/Bid Volume/Fill Empty Cells/CalculatePeaks
			
			public void AddBidAskVolume(int priceTick, int bidVol, int askVol)
			{
				_ticknumber++;
				if(_ticknumber == 1)BAfirstTickPrice = priceTick;
				
				_askLadder.Increment(priceTick,askVol,true);
				_bidLadder.Increment(priceTick,bidVol,true);
				
				_volumeLadder.Set(priceTick,_bidLadder.Cells[priceTick] + _askLadder.Cells[priceTick]);
				HighVolumePriceTicks(priceTick);    // this evaluate both High Volume and Prior High Volume stats.
				
				_deltaLadder.Set(priceTick,_askLadder.Cells[priceTick] - _bidLadder.Cells[priceTick]);
				_deltaLadder.ComputeMax();
				
				_diadeltaLadder.Set(priceTick-1,_askLadder.Cells[priceTick] - (_bidLadder.Cells.ContainsKey(priceTick-1) ? _bidLadder.Cells[priceTick-1] : 0));
				_diadeltaLadder.ComputeMax();
				
				if(bidVol == 0)_isLastTickAtBid=false;
				else _isLastTickAtBid = true;
				
				BAlastTickPrice=priceTick;
				_currentdelta += (askVol - bidVol);
				
				if (_currentdelta > _deltamax ) {_deltamax=_currentdelta; _maxdeltaticknumber = _ticknumber;}
				if (_currentdelta < _deltamin) {_deltamin=_currentdelta; _mindeltaticknumber = _ticknumber;}
				
				CalculateOpenStartFinish();
				_cdLadder.Set(priceTick,_currentdelta);
			}
			
			public void CalculateOpenStartFinish()
			{
				if(_ticknumber == 1 ) _opendelta = _currentdelta;
				
				//Start
				if ( MaxDTN < MinDTN ) 
				{
					if(_deltamax > _opendelta )_start = _deltamax - _opendelta;
					else if(_deltamax == _opendelta) _start = _deltamin - _opendelta;
				}
				else if ( MaxDTN > MinDTN )
				{
					if(_deltamin < _opendelta) _start =  _deltamin - _opendelta;
					else if(_deltamin == _opendelta) _start = _deltamax - _opendelta;
				}
				//else _start =0;
				else _start = _opendelta;
				
				//Finish
				if ( MaxDTN < MinDTN ) _finish = _currentdelta - _deltamin;
				else if ( MaxDTN > MinDTN ) _finish = _currentdelta - _deltamax;
				else _finish =0;
				
			}
			
			public void HighVolumePriceTicks(int priceTick)
			{
				
				if( _volumeLadder.Cells[priceTick] > _volumeLadder.MaxLadderValue)
				{
					if(_volumeLadder.Cells.Count > 1 && priceTick != _volumeLadder.MaxLadderValuePriceTick)
					{
						_volumeLadder.Max2LadderValue = _volumeLadder.MaxLadderValue;
						_volumeLadder.Max2LadderValuePriceTick = _volumeLadder.MaxLadderValuePriceTick;
					}
					
					_volumeLadder.MaxLadderValue = _volumeLadder.Cells[priceTick];
					_volumeLadder.MaxLadderValuePriceTick = priceTick;
				}
			}
			
			#endregion
			
			#region CalcLevelBA
			public void CalcLevelsBA(int pricetick)
			{
				int curlevel;			
				
				if (!CDLadder.Cells.ContainsKey(pricetick))
				{
					return;
				}
					
				int level0=CDLadder.Cells[pricetick];
				int level=level0;
				
				int i;
				int posmax=pricetick;
				
				for ( i=pricetick+1;i<=CDLadder.MaxPriceLevel;i++)
					if (CDLadder.Cells.ContainsKey(i))
					{
						curlevel=CDLadder.Cells[i];						
						if (curlevel>level)
						{

							level=curlevel;
							posmax=i;
						}
					}

				
				HighCOT=level0-level;
					
				level=level0;	
				int posmin=pricetick;
				for ( i=pricetick-1;i>=CDLadder.MinPriceLevel;i--)
					if (CDLadder.Cells.ContainsKey(i))
					{
						curlevel=CDLadder.Cells[i];						
						if (curlevel<level)
						{
							level=curlevel;
							posmin=i;
						}
					}

				
				LowCOT=level0-level;
	
			}
			#endregion
			
			//Graphical methods
			#region DrawCross
			private void DrawCross(float x)
			{
				if(!helper.ladderBAShowNumbers || !helper.showBALadder || helper.clt != CenterLadderType.BidXAsk)return;
				int curTick;
				float y;
				Font ft = new Font(helper.baFont.Name,helper.baFont.Size,FontStyle.Bold);
				foreach(KeyValuePair<int,int> cell in _bidLadder.Cells)
				{
					SolidBrush sb;
					curTick = cell.Key;
					y = helper.GetY(curTick);
					if(_bidLadder.Cells.ContainsKey(curTick) || _askLadder.Cells.ContainsKey(curTick))
					{
						
						if(BAlastTickPrice > BAfirstTickPrice)
						{
							if( curTick < BAfirstTickPrice || curTick > BAlastTickPrice)sb = new SolidBrush(helper.customCandleNeutralColor);
							else sb = new SolidBrush(helper.customCandleUpColor);
							helper.graphics.DrawString("X",ft,sb,x,y,centerSF);
						}
						else if( BAfirstTickPrice > BAlastTickPrice)
						{
							if( curTick > BAfirstTickPrice || curTick < BAlastTickPrice)sb = new SolidBrush(helper.customCandleNeutralColor);
							else sb = new SolidBrush(helper.customCandleDnColor);
							helper.graphics.DrawString("X",ft,sb,x,y,centerSF);
						}
						else if( BAfirstTickPrice == BAlastTickPrice)
						{
							if( curTick < BAfirstTickPrice || curTick > BAlastTickPrice)sb = new SolidBrush(helper.customCandleNeutralColor);
							else sb = new SolidBrush(helper.myTickPen.Color);
							helper.graphics.DrawString("X",ft,sb,x,y,centerSF);
						}
						
					}
				}
				
			}
			#endregion
			
			#region DrawVWAP
			private void DrawVWAP(float x)
			{
				if(!helper.showVWAP)return;
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				Rect.X = x;
				Rect.Height = helper.scalingfactor;
				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+2;
				int vwapTick = _cdLadder.helper.dsSessionVWAP.Get(_barIndex);
				
				Rect.Y = helper.GetY((float)vwapTick);
				Color c = helper.myVwapPen.Color;
				helper.graphics.FillRectangle(new SolidBrush(Color.FromArgb(50,c)),Rect);
				helper.graphics.DrawLine(helper.myVwapPen,Rect.X,Rect.Y,Rect.X+Rect.Width,Rect.Y);
				helper.graphics.DrawLine(helper.myVwapPen,Rect.X,Rect.Y+helper.scalingfactor,Rect.X+Rect.Width,Rect.Y+helper.scalingfactor);
				
			}
			#endregion
			
			#region Custom Candle/Body
			private void DrawCandleBody(float x, float xc,float bodyWidth, bool fillBody, int fillOpacity)
			{
				Pen pen;
				if(_ticknumber > 0)
				{
					if(BAlastTickPrice > BAfirstTickPrice)
					{
						pen = new Pen(helper.customCandleUpColor,helper.customCandleThickness);
						if(fillBody)helper.graphics.FillRectangle(new SolidBrush(Color.FromArgb(fillOpacity,helper.customCandleUpColor)),x,helper.GetY(BAlastTickPrice+0.5f),bodyWidth,helper.scalingfactor*(BAlastTickPrice-BAfirstTickPrice+1));
						helper.graphics.DrawRectangle(pen,x,helper.GetY(BAlastTickPrice+0.5f),bodyWidth,helper.scalingfactor*(BAlastTickPrice-BAfirstTickPrice+1));
						helper.graphics.DrawLine(pen,xc,helper.GetY(_bidLadder.MaxPriceLevel+0.5f),xc,helper.GetY(BAlastTickPrice+0.5f));
						helper.graphics.DrawLine(pen,xc,helper.GetY(BAfirstTickPrice-0.5f),xc,helper.GetY(_bidLadder.MinPriceLevel-0.5f));
					}
					else if(BAfirstTickPrice > BAlastTickPrice)
					{
						pen = new Pen(helper.customCandleDnColor,helper.customCandleThickness);
						if(fillBody)helper.graphics.FillRectangle(new SolidBrush(Color.FromArgb(fillOpacity,helper.customCandleDnColor)),x,helper.GetY(BAfirstTickPrice+0.5f),bodyWidth,helper.scalingfactor*(BAfirstTickPrice-BAlastTickPrice+1));
						helper.graphics.DrawRectangle(pen,x,helper.GetY(BAfirstTickPrice+0.5f),bodyWidth,helper.scalingfactor*(BAfirstTickPrice-BAlastTickPrice+1));
					    helper.graphics.DrawLine(pen,xc,helper.GetY(_bidLadder.MaxPriceLevel+0.5f),xc,helper.GetY(BAfirstTickPrice+0.5f));
						helper.graphics.DrawLine(pen,xc,helper.GetY(BAlastTickPrice-0.5f),xc,helper.GetY(_bidLadder.MinPriceLevel-0.5f));
					}
					else
					{
						pen = new Pen(helper.customCandleNeutralColor,helper.customCandleThickness);
						if(fillBody)helper.graphics.FillRectangle(new SolidBrush(Color.FromArgb(fillOpacity,helper.customCandleNeutralColor)),x,helper.GetY(BAlastTickPrice+0.5f),bodyWidth,helper.scalingfactor);
						helper.graphics.DrawRectangle(pen,x,helper.GetY(BAlastTickPrice+0.5f),bodyWidth,helper.scalingfactor);
						helper.graphics.DrawLine(pen,xc,helper.GetY(_bidLadder.MaxPriceLevel+0.5f),xc,helper.GetY(BAlastTickPrice+0.5f));
						helper.graphics.DrawLine(pen,xc,helper.GetY(BAlastTickPrice-0.5f),xc,helper.GetY(_bidLadder.MinPriceLevel-0.5f));
					}
				}
			}
			#endregion
			
			#region LastTickMark
			private void DrawLastTickMark(float x,int priceTick)
			{
				helper.graphics.DrawLine(helper.myTickPen,x,helper.GetY(priceTick+0.5f),x,helper.GetY(priceTick-0.5f));
			}
			#endregion
			
			#region Bottom Total Delta                 
			private void DrawBottomTotalDelta(float x, RowNumber rn)
			{	
				if(rn == RowNumber.None) return;
				
				if (_cdLadder.Cells.Count>0)
				{
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+2;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.X=x;
				Rect.Y=helper.bounds.Bottom-(float)rn-Rect.Height;
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
				
				#region Core
				int sum=_askLadder.Sum -_bidLadder.Sum;
				param.FillColor=Color.FromArgb(128,(sum>0)?helper.colorUp:helper.colorDown);// Colors Delta up/down
				
				string str1=LadderClass.Formatter(sum,helper.ladderBAFormat,helper);
				string s5="Δ= ";
				
			    param.FormattedData = (helper.abrev) ? s5 +str1 : str1;// Formats into a string
				#endregion
					
				
				DrawBottomBox(Rect,param,helper);
				if(!helper.legends.ContainsKey("Delta"))helper.legends.Add("Delta",(int)rn);
				}
			}
			#endregion              
			
			#region Bottom Cummulative Delta  // by devdas
			private void DrawBottomCumDelta(float x,RowNumber rn)
			{	
				if(rn == RowNumber.None) return;
				if (_cdLadder.Cells.Count>0)
				{
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+2;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.X=x;
				Rect.Y=helper.bounds.Bottom-(float)rn-Rect.Height;
				
				#region Core
				int sum =_cdLadder.helper.dsSessionCD.Get(_barIndex);
				
				param.FillColor=Color.FromArgb(128,(sum>0)?helper.colorUp:helper.colorDown);// Colors Delta up/down
				
				string str1=LadderClass.Formatter(sum,helper.ladderBAFormat,helper);
				string s5="CΔ= ";
			
				param.FormattedData = (helper.abrev) ? s5 + str1 : str1;
				#endregion 
					
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
				DrawBottomBox(Rect,param,helper);
				if(!helper.legends.ContainsKey("CumDelta"))helper.legends.Add("CumDelta",(int)rn);
				}
			}
			#endregion
			
			#region Bottom Total Volume
			private void DrawBottomVolume(float x,RowNumber rn)
			{	
				if(rn == RowNumber.None) return;
				if (_cdLadder.Cells.Count>0)
				{
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+2;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.X=x;
				Rect.Y=helper.bounds.Bottom-(float)rn-Rect.Height;
				
				#region Core
				int sum=_askLadder.Sum+_bidLadder.Sum;//Volume
				string s5="V= ";
				param.FillColor=helper.colorSideLadderVolume;
				
				string str1=LadderClass.Formatter((helper.ladderBANoMinus)?Math.Abs(sum):sum,helper.ladderBAFormat,helper);
				
				param.FormattedData = (helper.abrev) ? s5 +str1 : str1;
				#endregion
					
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
				DrawBottomBox(Rect,param,helper);
				if(!helper.legends.ContainsKey("Volume"))helper.legends.Add("Volume",(int)rn);
				}
			}
			#endregion
			
			#region Bottom Cummulative Volume    // by devdas Cummulative session Volume
			private void DrawBottomCumVolume(float x,RowNumber rn)
			{	
				if(rn == RowNumber.None) return;
				if (_cdLadder.Cells.Count>0)
				{
				
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+2;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.X=x;
				Rect.Y=helper.bounds.Bottom-(float)rn-Rect.Height;
				
				#region Core
				int sum =_cdLadder.helper.dsSessionCV.Get(_barIndex);
			
				string s5="cV= ";
				param.FillColor=helper.colorSideLadderVolume;
				
				string str1=LadderClass.Formatter((helper.ladderBANoMinus)?Math.Abs(sum):sum,helper.ladderBAFormat,helper);
				
				param.FormattedData = (helper.abrev) ? s5 +str1 : str1;
				#endregion	
				
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
				DrawBottomBox(Rect,param,helper);
				if(!helper.legends.ContainsKey("CumVolume"))helper.legends.Add("CumVolume",(int)rn);
				}
			}
			#endregion
			
			#region Bottom CumDelta Percent On CumVol 
			private void DrawBottomCumDeltaPercent(float x,RowNumber rn)// by devdas cum delta percent on cum session volume
			{	
				if(rn == RowNumber.None) return;
				if (_cdLadder.Cells.Count>0)
				{
				
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+2;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.X=x;
				Rect.Y=helper.bounds.Bottom-(float)rn-Rect.Height;
				
				#region Core
				decimal Delta=	_cdLadder.helper.dsSessionCD.Get(_barIndex);
				decimal Volume=	_cdLadder.helper.dsSessionCV.Get(_barIndex);
				
				decimal sum4 =  (decimal)  (100*Delta/Volume);// decimal version of delta % sent to st6
				int sum3= (int)  (100*Delta/Volume);// int used for param fill color
				if (sum3>3)
				param.FillColor=Color.FromArgb(128,(Volume>0)?helper.colorUp:Color.Transparent); // Sets threshold for green coloring ex 3%
				
				if (sum3<-3)
				param.FillColor=Color.FromArgb(128,(Volume>0)?helper.colorDown:Color.Transparent);// Sets threshold for red coloring ex -3%
				
				string s5="";
				string st6=String.Format("{0:0.0}",sum4);//formating of decimal to 0 places
				s5="cΔp= ";
				
				param.FormattedData = (helper.abrev) ? s5 +st6+"%" : st6+"%";
				#endregion
				
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
				DrawBottomBox(Rect,param,helper);
				if(!helper.legends.ContainsKey("CumDelta %"))helper.legends.Add("CumDelta %",(int)rn);
				}
			}
			#endregion
			
			
			#region Bottom Sellers Percent
			private void DrawBottomSellersPercent(float x, RowNumber rn)// Sellers %
			{	
				if(rn == RowNumber.None) return;
				if (_cdLadder.Cells.Count>0)
				{
				
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+2;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.X=x;
				Rect.Y=helper.bounds.Bottom-(float)rn-Rect.Height;
				
				#region Core
				int AskSum=_askLadder.Sum;
				int BidSum=_bidLadder.Sum;
				decimal Volume=	_askLadder.Sum+_bidLadder.Sum;
			
				int AskVolume=_askLadder.Sum;
				int BidVolume=_bidLadder.Sum;
				
				decimal Delta=	_askLadder.Sum-_bidLadder.Sum;//gets decimal value of Delta
				
				decimal sum  =  (decimal)  (100*AskVolume/Volume);//%
				
				int sum3 =  (int)  (100*Delta/Volume);
					
				decimal sum2 =  (decimal)  (100*BidVolume/Volume);
				
				param.FillColor=Color.FromArgb(128,(AskSum>BidSum)?helper.colorUp:helper.colorDown);//Coloring of the total derived
				
				string str1=String.Format("{0:0.0}",sum2);//formating of decimal to 1 places
				string s5="S= ";
				param.FormattedData = (helper.abrev) ? s5 +str1+"%" : str1+"%";
				#endregion
				
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
				DrawBottomBox(Rect,param,helper);
				if(!helper.legends.ContainsKey("Sellers %"))helper.legends.Add("Sellers %",(int)rn);
				}
			}
			#endregion
			
			#region Bottom Buyers Percent
			private void DrawBottomBuyersPercent(float x, RowNumber rn)//buyers % control added timmyb
			{	
				if(rn == RowNumber.None) return;
				if (_cdLadder.Cells.Count>0)
				{
				
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+2;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.X=x;
				Rect.Y=helper.bounds.Bottom-(float)rn-Rect.Height;
				
				#region Core
				int AskVolume=_askLadder.Sum;
				int BidVolume=_bidLadder.Sum;
				
				decimal Delta=	_askLadder.Sum-_bidLadder.Sum;//gets decimal value of Delta
				decimal Volume=	_askLadder.Sum+_bidLadder.Sum;//gets decimal value of Volume
				decimal sum  =  (decimal)  (100*AskVolume/Volume);//%
				
				int sum2 =  (int)  (100*BidVolume/Volume);
				int AskSum= _askLadder.Sum;	
				int BidSum= _bidLadder.Sum;
				int sum3 =  (int)  (100*Delta/Volume);
				
				param.FillColor=Color.FromArgb(128,(AskSum>BidSum)?helper.colorUp:helper.colorDown);
				
				string s5="B= ";
				string str1=String.Format("{0:0.0}",sum);//formating of decimal to 1 places
				param.FormattedData = (helper.abrev) ? s5 +str1+"%" : str1+"%";
				#endregion
				
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
				DrawBottomBox(Rect,param,helper);
				if(!helper.legends.ContainsKey("Buyers %"))helper.legends.Add("Buyers %",(int)rn);
				}
			}
			#endregion
			
			
			#region Bottom Total Seller
			private void DrawBottomSellers(float x,RowNumber rn)//total contracts sold
			{	
				if(rn == RowNumber.None) return;
				if (_cdLadder.Cells.Count>0)
				{
				
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+2;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.X=x;
				Rect.Y=helper.bounds.Bottom-(float)rn-Rect.Height;
				
				#region Core
				int Delta=	_askLadder.Sum-_bidLadder.Sum;
				int Volume=	_askLadder.Sum+_bidLadder.Sum;
				int AskVolume=_askLadder.Sum;
				int BidVolume=_bidLadder.Sum;
				
				int sum  =  (int)  (100*AskVolume/Volume);//%
				int sum2 =  (int)  (100*BidVolume/Volume);
				int AskSum= _askLadder.Sum;	
				int BidSum= _bidLadder.Sum;
				int sum3 =  (int)  (100*Delta/Volume);
				
				param.FillColor=Color.FromArgb(128,(AskSum>BidSum)?helper.colorUp:helper.colorDown);
				string st1=LadderClass.Formatter((helper.ladderBANoMinus)?Math.Abs(sum):sum,helper.ladderBAFormat,helper);
				string st2=LadderClass.Formatter((helper.ladderBANoMinus)?Math.Abs(AskSum):AskSum,helper.ladderBAFormat,helper);// %
				string st3=LadderClass.Formatter((helper.ladderBANoMinus)?Math.Abs(BidSum):BidSum,helper.ladderBAFormat,helper);
				string st4=LadderClass.Formatter((helper.ladderBANoMinus)?Math.Abs(BidVolume):BidVolume,helper.ladderBAFormat,helper);
				string st5=LadderClass.Formatter((helper.ladderBANoMinus)?Math.Abs(sum3):sum3,helper.ladderBAFormat,helper);
				
				string s5="S= ";	
				param.FormattedData = (helper.abrev) ? s5 +st4 : st4;
				#endregion
				
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
				DrawBottomBox(Rect,param,helper);
				if(!helper.legends.ContainsKey("Seller"))helper.legends.Add("Seller",(int)rn);
				}
			}
			#endregion
			
			#region Bottom Total Buyer
			private void DrawBottomBuyers(float x,RowNumber rn)//total contracts bought Buyers
			{	
				if(rn == RowNumber.None) return;
				if (_cdLadder.Cells.Count>0)
				{
				
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+2;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.X=x;
				Rect.Y=helper.bounds.Bottom-(float)rn-Rect.Height;
				
				#region Core
				int Delta=	_askLadder.Sum-_bidLadder.Sum;
				int Volume=	_askLadder.Sum+_bidLadder.Sum;
				int AskVolume=_askLadder.Sum;
				int BidVolume=_bidLadder.Sum;
				
				int sum  =  (int)  (100*AskVolume/Volume);//%
				int sum2 =  (int)  (100*BidVolume/Volume);
				int AskSum= _askLadder.Sum;	
				int BidSum= _bidLadder.Sum;
				int sum3 =  (int)  (100*Delta/Volume);
				
				param.FillColor=Color.FromArgb(128,(AskSum>BidSum)?helper.colorUp:helper.colorDown);
				
				string st1=LadderClass.Formatter((helper.ladderBANoMinus)?Math.Abs(sum):sum,helper.ladderBAFormat,helper);
				string st2=LadderClass.Formatter((helper.ladderBANoMinus)?Math.Abs(AskSum):AskSum,helper.ladderBAFormat,helper);// %
				string st3=LadderClass.Formatter((helper.ladderBANoMinus)?Math.Abs(BidSum):BidSum,helper.ladderBAFormat,helper);
				string st4=LadderClass.Formatter((helper.ladderBANoMinus)?Math.Abs(AskVolume):AskVolume,helper.ladderBAFormat,helper);
				string st5=LadderClass.Formatter((helper.ladderBANoMinus)?Math.Abs(sum3):sum3,helper.ladderBAFormat,helper);
				string s5="B= ";
				param.FormattedData = (helper.abrev) ? s5 +st4 : st4;
				#endregion
				
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
				DrawBottomBox(Rect,param,helper);
				if(!helper.legends.ContainsKey("Buyer"))helper.legends.Add("Buyer",(int)rn);
				}
			}
			#endregion
			
			
			#region Bottom MinDelta Percent
			private void DrawBottomMinDeltaPercent(float x, RowNumber rn)//Min Delta % control added timmyb
			{	
				if(rn == RowNumber.None) return;
				if (_cdLadder.Cells.Count>0)
				{
				
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+2;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.X=x;
				Rect.Y=helper.bounds.Bottom-(float)rn-Rect.Height;
				
				#region Core
				int AskVolume=_askLadder.Sum;
				int BidVolume=_bidLadder.Sum;
				
				decimal Delta=	_askLadder.Sum-_bidLadder.Sum;//gets decimal value of Delta
				decimal Volume=	_askLadder.Sum+_bidLadder.Sum;//gets decimal value of Volume
				decimal sum  =  (decimal)  (100*_deltamin/Volume);//%
				
				int sum2 =  (int)  (100*BidVolume/Volume);
				int AskSum= _askLadder.Sum;	
				int BidSum= _bidLadder.Sum;
				int sum3 =  (int)  (100*Delta/Volume);
				
				param.FillColor=Color.FromArgb(128,(sum>0)?helper.colorUp:helper.colorDown);
				string str1=String.Format("{0:0.0}",sum);//formating of decimal to 1 places
				string s5="Δ- = ";
				param.FormattedData = (helper.abrev) ? s5 +str1+"%" : str1+"%";
				#endregion
				
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
				DrawBottomBox(Rect,param,helper);
				if(!helper.legends.ContainsKey("MinDelta %"))helper.legends.Add("MinDelta %",(int)rn);
				}
			}
			#endregion
			
			#region Bottom MaxDelta Percent
			private void DrawBottomMaxDeltaPercent(float x,RowNumber rn)//Max Delta % control added timmyb
			{	
				if(rn == RowNumber.None) return;
				if (_cdLadder.Cells.Count>0)
				{
				
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+2;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.X=x;
				Rect.Y=helper.bounds.Bottom-(float)rn-Rect.Height;
				
				#region Core
				int AskVolume=_askLadder.Sum;
				int BidVolume=_bidLadder.Sum;
				
				decimal Delta=	_askLadder.Sum-_bidLadder.Sum;//gets decimal value of Delta
				decimal Volume=	_askLadder.Sum+_bidLadder.Sum;//gets decimal value of Volume
				decimal sum  =  (decimal)  (100*_deltamax/Volume);//%
				
				int sum2 =  (int)  (100*BidVolume/Volume);
				int AskSum= _askLadder.Sum;	
				int BidSum= _bidLadder.Sum;
				int sum3 =  (int)  (100*Delta/Volume);
				
				param.FillColor=Color.FromArgb(128,(sum>0)?helper.colorUp:helper.colorDown);
				string s5="Δ+ = ";
				string str1=String.Format("{0:0.0}",sum);//formating of decimal to 1 places
				param.FormattedData = (helper.abrev) ? s5 +str1+"%" : str1+"%";
				#endregion
				
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
				DrawBottomBox(Rect,param,helper);
				if(!helper.legends.ContainsKey("MaxDelta %"))helper.legends.Add("MaxDelta %",(int)rn);
				}
			}
			#endregion
			
			#region Bottom Delta Percent
			private void DrawBottomDeltaPercent(float x,RowNumber rn)//Delta % control added timmyb
			{	
				if(rn == RowNumber.None) return;
				if (_cdLadder.Cells.Count>0)
				{
				
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+2;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.X=x;
				Rect.Y=helper.bounds.Bottom-(float)rn-Rect.Height;
				
				#region Core
				decimal Delta=	_askLadder.Sum-_bidLadder.Sum;//gets decimal value of Delta
				decimal Volume=	_askLadder.Sum+_bidLadder.Sum;//gets decimal value of Volume
				
				decimal sum4 =  (decimal)  (100*Delta/Volume);// decimal version of delta % sent to st6
				int sum3= (int)  (100*Delta/Volume);// int used for param fill color
				
				if (sum3>20)
				param.FillColor=Color.FromArgb(128,(Volume>0)?helper.colorUp:Color.Transparent); // Sets threshold for green coloring ex 20%
				
				if (sum3<-20)
				param.FillColor=Color.FromArgb(128,(Volume>0)?helper.colorDown:Color.Transparent);// Sets threshold for red coloring ex -20%
				
				string s5="Δ= ";
				string st6=String.Format("{0:0.0}",sum4);//formating of decimal to 2 places
				param.FormattedData = (helper.abrev) ? s5 +st6+"%" : st6+"%";
				#endregion
				
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
				DrawBottomBox(Rect,param,helper);
				if(!helper.legends.ContainsKey("Delta %"))helper.legends.Add("Delta %",(int)rn);
				}
			}
			#endregion
			
			
			#region Bottom MinDelta
			private void DrawBottomMinDelta(float x,RowNumber rn)//Mindelta
			{	
				if(rn == RowNumber.None) return;
				if (_cdLadder.Cells.Count>0)
				{
				
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+2;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.X=x;
				Rect.Y=helper.bounds.Bottom-(float)rn-Rect.Height;
				
				#region Core
				int sum=_deltamin;
				param.FillColor=Color.FromArgb(128,(sum>0)?helper.colorUp:helper.colorDown);
				
				string s5="Δ- = ";
			    string str1=LadderClass.Formatter(sum,helper.ladderBAFormat,helper);
				param.FormattedData = (helper.abrev) ? s5 +str1 : str1;
				#endregion
				
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
				DrawBottomBox(Rect,param,helper);
				if(!helper.legends.ContainsKey("MIN Delta"))helper.legends.Add("MIN Delta",(int)rn);
				}
			}
			#endregion
			
			#region Bottom MaxDelta
			private void DrawBottomMaxDelta(float x,RowNumber rn)//Max delta
			{	
				if(rn == RowNumber.None) return;
				if (_cdLadder.Cells.Count>0)
				{
				
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+1;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.X=x;
				Rect.Y=helper.bounds.Bottom-(float)rn-Rect.Height;
				
				#region Core
				string s5="Δ+ = ";
				int sum=_deltamax;
				param.FillColor=Color.FromArgb(128,(sum>0)?helper.colorUp:helper.colorDown);
				//string str1=LadderClass.Formatter((helper.ladderBANoMinus)?Math.Abs(sum):sum,helper.ladderBAFormat,helper);
				string str1=LadderClass.Formatter(sum,helper.ladderBAFormat,helper);
				param.FormattedData = (helper.abrev) ? s5 +str1 : str1;
				#endregion
				
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
				DrawBottomBox(Rect,param,helper);
				if(!helper.legends.ContainsKey("MAX Delta"))helper.legends.Add("MAX Delta",(int)rn);
			
				}
			}
			#endregion
			
			
			#region Bottom Finish
			private void DrawBottomFinish(float x,RowNumber rn)
			{
				if(rn == RowNumber.None) return;
				if (_cdLadder.Cells.Count>0)
				{
				
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+1;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.X=x;
				Rect.Y=helper.bounds.Bottom-(float)rn-Rect.Height;
				
				#region Core
				string s5="F = ";
				param.FillColor=Color.FromArgb(128,(_finish>0)?helper.colorUp:helper.colorDown);
				string str1=LadderClass.Formatter(_finish,helper.ladderBAFormat,helper);
				param.FormattedData = (helper.abrev) ? s5 +str1 : str1;
				#endregion
				
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
				DrawBottomBox(Rect,param,helper);
				if(!helper.legends.ContainsKey("Finish"))helper.legends.Add("Finish",(int)rn);
			
				}
			}
			#endregion
			
			#region Bottom Start
			private void DrawBottomStart(float x,RowNumber rn)
			{
				if(rn == RowNumber.None) return;
				if (_cdLadder.Cells.Count>0)
				{
				
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+1;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.X=x;
				Rect.Y=helper.bounds.Bottom-(float)rn-Rect.Height;
				
				#region Core
				string s5="F = ";
				param.FillColor=Color.FromArgb(128,(_start>0)?helper.colorUp:helper.colorDown);
				string str1=LadderClass.Formatter(_start,helper.ladderBAFormat,helper);
				param.FormattedData = (helper.abrev) ? s5 +str1 : str1;
				#endregion
				
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
				DrawBottomBox(Rect,param,helper);
				if(!helper.legends.ContainsKey("Start"))helper.legends.Add("Start",(int)rn);
			
				}
			}
			#endregion 
			
			#region Bottom UpReversal
			private void DrawBottomUpReversal(float x)
			{	
				if (_cdLadder.Cells.Count>0)
				{
					
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+1;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.X=x;
				Rect.Y=helper.GetY(_cdLadder.MinPriceLevel-1.0f);
				
				int sum=_currentdelta - _deltamin;
				param.FillColor=Color.FromArgb(128,(sum>0)?helper.colorUp:helper.colorDown);
				string st1=LadderClass.Formatter((helper.ladderBANoMinus)?Math.Abs(sum):sum,helper.ladderBAFormat,helper);
				param.FormattedData="CR^ "+st1;
				if (sum > helper.contractReversalSetting)
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
					
				if (sum > helper.contractReversalSetting)	
				DrawCOTAndRevCell(Rect,param,helper);
				}
			
				}
			#endregion
			
			#region Bottom DnReversal
			private void DrawBottomDnReversal(float x)
			{	
				if (_cdLadder.Cells.Count>0)
				{
				
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
				DrawData param=new DrawData(helper);

				Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+1;
				Rect.Height=2f+helper.bottomFont.GetHeight();
				Rect.Y=helper.GetY(_cdLadder.MaxPriceLevel+2.0f);
				Rect.X=x;
				
				int sum=_currentdelta - _deltamax;
				param.FillColor=Color.FromArgb(128,(sum>0)?helper.colorUp:helper.colorDown);
				string st1=LadderClass.Formatter((helper.ladderBANoMinus)?Math.Abs(sum):sum,helper.ladderBAFormat,helper);
				param.FormattedData="CR "+st1;
				if (sum < -helper.contractReversalSetting)	
				helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
					
				if (sum < -helper.contractReversalSetting)	
				DrawCOTAndRevCell(Rect,param,helper);
			
				}
					
			}
			#endregion
			
			
			#region Draw COTs
			private void DrawCOTs(float x)
			{		
			try{
				
				
		        if (_cdLadder.Cells.Count>0)
				{
				RectangleF Rect=new RectangleF(0f,0f,0f,0f);
					DrawData param=new DrawData(helper);
					
					Rect.Height = 1+helper.baFont.GetHeight();
					Rect.X=x;
                    Rect.Width=2*(helper.BAcellwidth+helper.chartcontrol.BarWidth)+2;
				
					Rect.Y= (float)((-helper.baFont.GetHeight()))+ helper.GetY(_cdLadder.MaxPriceLevel+(float)+1f);
					param.FillColor=helper.colorDown;
					
					param.FormattedData=LadderClass.Formatter((helper.ladderBANoMinus)?Math.Abs(HighCOT):HighCOT,helper.ladderBAFormat,helper);
					helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
					DrawCOTAndRevCell(Rect,param,helper);
					
					Rect.Y=helper.GetY(_cdLadder.MinPriceLevel-1.0f);
					param.FillColor=helper.colorUp;
					param.FormattedData=LadderClass.Formatter(LowCOT,helper.ladderBAFormat,helper);
					helper.graphics.FillRectangle(helper.myWhiteBrush,Rect);
					DrawCOTAndRevCell(Rect,param,helper);
				}
				} 
			catch (Exception ex)
			{
			
				return;
			}	
			
			}
			#endregion
			
			#region Draw Ladder And Bottom Boxes
            public void DrawLadder()
				
			{	
				try
				{
				float xbid,xask,xcenter;			
				float cellwidth=Math.Max(10f,helper.BAcellwidth);
				float bothcellwidth = Math.Max(20f,2*helper.BAcellwidth+2*helper.chartcontrol.BarWidth);
				
				ChartControl CC=helper.chartcontrol;
				LadderClass DiagonalDeltaLadder;
				
				xbid=CC.GetXByBarIdx(helper.bars,_barIndex)-CC.BarWidth-helper.BAcellwidth-1f;//changes to create blank space between histogram and candle outline
				xask=CC.GetXByBarIdx(helper.bars,_barIndex)+CC.BarWidth+1f;
				xcenter = CC.GetXByBarIdx(helper.bars,_barIndex);
				
			    //float spacer = bothcellwidth/10.0f;
				float spacer = 0.1f*bothcellwidth;
					
				if(helper.showVWAP)
					DrawVWAP(xbid);
				
				if (helper.showBALadder)
				{
					float minw=0f;
					switch(helper.clt){
						case CenterLadderType.BidAndAsk:
							
					        #region Bid Ask Display
						_bidLadder.FillCells=BidLadderFillParams;
				        _askLadder.FillCells=AskLadderFillParams;
							
						_bidLadder.EmptyLadder(xbid+0.5f*spacer,cellwidth-0.8f*spacer);
						_askLadder.EmptyLadder(xask+0.3f*spacer,cellwidth-0.8f*spacer);
						
						_bidLadder.DrawGeneralLadderFillCells(xbid+0.5f*spacer,cellwidth-0.8f*spacer,Direction.RightToLeft,helper.showBordersBA,true,helper.ladderBAShowNumbers);
						_askLadder.DrawGeneralLadderFillCells(xask+0.3f*spacer,cellwidth-0.8f*spacer,Direction.LeftToRight,helper.showBordersBA,true,helper.ladderBAShowNumbers);	
						
						if (_askLadder.Sum>_bidLadder.Sum && helper.ladderBAShowOuterBoarder == true)
							_askLadder.DrawBorder(xask, cellwidth,new Pen(helper.colorUp,2));
						else if (_bidLadder.Sum>_askLadder.Sum && helper.ladderBAShowOuterBoarder == true)
							_bidLadder.DrawBorder(xbid,cellwidth,new Pen(helper.colorDown,2));
						
						DrawCandleBody(xcenter-0.2f*spacer,xcenter,0.4f*spacer,true,100);
						
						if (helper.ladderBAShowMaxLevel)
							_bidLadder.DrawCell(xbid+0.5f*spacer,bothcellwidth-spacer+1,_volumeLadder.MaxLadderValuePriceTick,helper.myVolPen);
						
						if(_isLastTickAtBid)DrawLastTickMark(xbid+(0.1f*spacer),BAlastTickPrice);
							else DrawLastTickMark(xask+cellwidth-(0.1f*spacer),BAlastTickPrice);
						break;
						#endregion
						
						case CenterLadderType.BidXAsk:
							
							#region Bid X Ask Display
						_bidLadder.FillCells=BidLadderFillParams;
				        _askLadder.FillCells=AskLadderFillParams;
							
						_bidLadder.EmptyLadder(xbid+0.5f*spacer,cellwidth-0.8f*spacer);
						_askLadder.EmptyLadder(xask+0.3f*spacer,cellwidth-0.8f*spacer);
						
						_bidLadder.DrawGeneralLadderFillCells(xbid+0.5f*spacer,cellwidth-0.8f*spacer,Direction.RightToLeft,helper.showBordersBA,true,helper.ladderBAShowNumbers);
						_askLadder.DrawGeneralLadderFillCells(xask+0.3f*spacer,cellwidth-0.8f*spacer,Direction.LeftToRight,helper.showBordersBA,true,helper.ladderBAShowNumbers);
						
						_bidLadder.DrawCell(xbid+0.5f*spacer,bothcellwidth-spacer+1,_volumeLadder.MaxLadderValuePriceTick,helper.myVolPen);
						
						if(_isLastTickAtBid)DrawLastTickMark(xbid+(0.1f*spacer),BAlastTickPrice);
							else DrawLastTickMark(xask+cellwidth-(0.1f*spacer),BAlastTickPrice);
						if(helper.clt == CenterLadderType.BidXAsk)DrawCross(xcenter);
						break;
						#endregion
						
						case CenterLadderType.VolAndDelta:
							
						    #region VolAndDelta Display
						_volumeLadder.FillCells = VolumeFillParams;
						_volumeLadder.EmptyLadder(xbid+0.5f*spacer,cellwidth-0.8f*spacer);
						_volumeLadder.DrawGeneralLadderFillCells(xbid+0.5f*spacer,cellwidth-0.8f*spacer,Direction.RightToLeft,helper.showBordersBA,true,helper.ladderBAShowNumbers);
						
						_deltaLadder.FillCells=DeltaFillParams;
						_deltaLadder.EmptyLadder(xask+0.3f*spacer,cellwidth-0.8f*spacer);
						_deltaLadder.DrawGeneralLadderFillCells(xask+0.3f*spacer,cellwidth-0.8f*spacer,Direction.LeftToRight,helper.showBordersBA,true,helper.ladderBAShowNumbers);
						
						DrawCandleBody(xcenter-0.2f*spacer,xcenter,0.4f*spacer,true,100);
							
						if(_volumeLadder.Max2LadderValue!=Int32.MinValue)
								_bidLadder.DrawCell(xbid+0.5f*spacer,bothcellwidth-spacer+1,_volumeLadder.Max2LadderValuePriceTick,helper.myVol2Pen);
						
						_volumeLadder.DrawCell(xbid+0.5f*spacer,bothcellwidth-spacer+1,_volumeLadder.MaxLadderValuePriceTick,helper.myVolPen);
						break;
						#endregion
						
						case CenterLadderType.CustomLadder:
							
							#region CustomLadder Display
							_volumeLadder.FillCells=CustomLadderFillParams;
						    _volumeLadder.EmptyLadder(xbid+spacer,bothcellwidth-2*spacer);
							_volumeLadder.DrawGeneralLadderFillCells(xbid+spacer,bothcellwidth-2*spacer,helper.clDirection,helper.showBordersBA,true,helper.ladderBAShowNumbers);
							
							if(helper.candleDirection == Direction.LeftToRight)DrawCandleBody(xbid+0.1f*spacer,xbid+0.3f*spacer,0.4f*spacer,true,100);
							else DrawCandleBody(xask+cellwidth-0.6f*spacer,xask+cellwidth-0.4f*spacer,0.4f*spacer,true,100);
							
							if (helper.cld == CustomLadderData.Volume || helper.cld == CustomLadderData.BidxAsk)
								_volumeLadder.DrawCell(xbid+spacer,bothcellwidth-2*spacer,_volumeLadder.MaxLadderValuePriceTick,helper.myVolPen);
							else if (helper.cld == CustomLadderData.Delta)
								_volumeLadder.DrawCell(xbid+spacer,bothcellwidth-2*spacer,_deltaLadder.MaxLadderValuePriceTick,helper.myVolPen);
							break;
							#endregion
						
						case CenterLadderType.DiagonalBidAsk:
							
							#region DiagonalBidAsk Display
							_bidLadder.FillCells = DiagonalBidLadderFillParams;
							_askLadder.FillCells = DiagonalAskLadderFillParams;
							
							_bidLadder.EmptyLadder(xbid+spacer,cellwidth-spacer);
						    _askLadder.EmptyLadder(xask,cellwidth-spacer);
							
							DrawCandleBody(xbid+spacer,xcenter,bothcellwidth-2*spacer+2,true,helper.customCandleFillOpacity);
						    
							_bidLadder.DrawGeneralLadderFillCells(xbid+spacer,cellwidth-spacer,Direction.RightToLeft,helper.showBordersBA,false,helper.ladderBAShowNumbers);	
						    _askLadder.DrawGeneralLadderFillCells(xask,cellwidth-spacer,Direction.LeftToRight,helper.showBordersBA,false,helper.ladderBAShowNumbers);
							
							if(_volumeLadder.Max2LadderValue!=Int32.MinValue)
								_bidLadder.DrawCell(xbid+spacer,bothcellwidth-2*spacer+1,_volumeLadder.Max2LadderValuePriceTick,helper.myVol2Pen);
							_bidLadder.DrawCell(xbid+spacer,bothcellwidth-2*spacer+1,_volumeLadder.MaxLadderValuePriceTick,helper.myVolPen);
							
							if(_isLastTickAtBid)DrawLastTickMark(xbid+(0.5f*spacer),BAlastTickPrice);
							else DrawLastTickMark(xask+cellwidth-(0.5f*spacer),BAlastTickPrice);
							break;
							#endregion
							
						case CenterLadderType.DiagonalDelta:
							
							#region DiagonalDelta Display
							
							_diadeltaLadder.FillCells = DeltaFillParams;
							
							_diadeltaLadder.EmptyLadder(xbid+spacer,bothcellwidth-2*spacer);
							_diadeltaLadder.DrawGeneralLadderFillCells(xbid+spacer,bothcellwidth-2*spacer,Direction.LeftToRight,helper.showBordersBA,true,helper.ladderBAShowNumbers);
							
							DrawCandleBody(xbid+0.1f*spacer,xbid+0.3f*spacer,0.4f*spacer,true,100);
							
							if (helper.ladderBAShowMaxLevel)
							_diadeltaLadder.DrawCell(xbid+spacer,bothcellwidth-2*spacer,_diadeltaLadder.MaxLadderValuePriceTick,helper.myVolPen);
							break;
							#endregion
						
					}
				}
				
				if (helper.ladderBAShowCOT)
					DrawCOTs(xbid);
					
				if (helper.bottomview == BottomViewType.Statistic)// show bottom boxes = true
				{
					DrawBottomTotalDelta(xbid,helper.deltaRow);
					DrawBottomCumDelta(xbid,helper.cumDeltaRow); 
				    DrawBottomVolume(xbid,helper.volRow);
					DrawBottomCumVolume(xbid,helper.cumVolRow);
					DrawBottomCumDeltaPercent(xbid,helper.pctCumDeltaRow);
					DrawBottomSellersPercent(xbid,helper.pctSellerRow);
					DrawBottomBuyersPercent(xbid,helper.pctBuyerRow);
					DrawBottomSellers(xbid,helper.sellerRow);
					DrawBottomBuyers(xbid,helper.buyerRow);
					DrawBottomMinDeltaPercent(xbid,helper.pctMinDeltaRow);
					DrawBottomMaxDeltaPercent(xbid,helper.pctMaxDeltaRow);
					DrawBottomDeltaPercent(xbid,helper.pctDeltaRow);
					DrawBottomMinDelta(xbid,helper.minDeltaRow);
					DrawBottomMaxDelta(xbid,helper.maxDeltaRow);
					DrawBottomFinish(xbid,helper.finishRow);
					DrawBottomStart(xbid,helper.startRow);
				}
				
				if (helper.ladderBAShowUpReversal)
					DrawBottomUpReversal(xbid);
					
				if (helper.ladderBAShowDwnReversal)
					DrawBottomDnReversal(xbid);
				
				
			    } catch (Exception ex){
				        return;	}
				
			}
			#endregion
			
			#region DrawData and Fill Params
			void GetRatioAndColor(int volume,int maxValue, ref float ratio, ref Color color, int priceTick)
			{
				try{
					int v = Math.Abs(volume);
				    int mv = Math.Abs(maxValue);
				    if (maxValue==0) ratio=0.0F;
				
				switch(helper.ccs){
					
					case CellColorScheme.MaximumOfLadder:
						ratio= (maxValue == 0) ? 0.0f :(float)Math.Abs(volume)/(float)maxValue;
						color=Color.FromArgb((int)(255*ratio),color);
						break;
					case CellColorScheme.FixedNumber:
						ratio=Math.Min(1f,(float)Math.Abs(volume)/(float)helper.ladderBAFixedNumber);
						color=Color.FromArgb((int)(255*ratio),color);
						break;
					case CellColorScheme.DominateSide:
						int dominantTotVol;
						if(helper.clt == CenterLadderType.BidAndAsk || helper.clt == CenterLadderType.BidXAsk)
						{
						dominantTotVol = (Math.Abs(_bidLadder.Cells[priceTick])+Math.Abs(_askLadder.Cells[priceTick]));
						color = (_bidLadder.Cells[priceTick] < _askLadder.Cells[priceTick]) ? helper.colorUp : helper.colorDown;
						ratio = (float)(dominantTotVol)/(float)(_volumeLadder.MaxLadderValue);
						}
						else ratio=(maxValue == 0) ? 0.0f :(float)Math.Abs(volume)/(float)maxValue;
						
						color = Color.FromArgb((int)(255*ratio),color);
						break;
				}
				
				if(helper.ladderBAHighSideMode)
				{
				if(v < (int)((helper.ladderBAHighSidePercent*mv/100.0)))color=helper.colorBackGround;
				}
				
				
				} catch (Exception ex){
				return;	}

			}
			
			DrawData BasicFillParams(int volume, int maxValue,Color color,int priceTick)
			{
				DrawData returnData=new DrawData(helper);
				
				float ratio=0.0F;
				returnData.FillColor=color;
				GetRatioAndColor(volume,maxValue,ref ratio,ref returnData.FillColor,priceTick);
				
				if (helper.cftCenter == CellFillType.Histogram)returnData.FillEndRatio=ratio;
				else if(helper.cftCenter == CellFillType.Full) returnData.FillEndRatio = 1;
				else if(helper.cftCenter == CellFillType.None) returnData.FillEndRatio = 0;
				
				if (helper.ladderBAShowNumbers)
					returnData.FormattedData=LadderClass.Formatter(volume,helper.ladderBAFormat,helper);
				
				if(Math.Abs(volume) == Math.Abs(maxValue))returnData.CurrFont = returnData.BABoldFont;
				else returnData.CurrFont = returnData.BAFont;
				return returnData;
			
			}
			
			DrawData DiagonalBidLadderFillParams(int volume,int maxValue,int priceTick)
			{
				DrawData returnData = BasicFillParams(volume,maxValue,helper.colorDown,priceTick);
				
				int N = helper.diaNumerator;
				int D = helper.diaDenominator;
				
				if(!_askLadder.Cells.ContainsKey(priceTick+1)) returnData.CurrBrush = (_bidLadder.Cells[priceTick] > helper.diagonalThresholdVolume) ? returnData.DiaStrongBidBrush : returnData.DiaNeutralBrush;
				else if(D*_bidLadder.Cells[priceTick] < N*_askLadder.Cells[priceTick+1] && _askLadder.Cells[priceTick+1] > helper.diagonalThresholdVolume)returnData.CurrBrush = returnData.DiaWeakBrush;
				else if (N*_bidLadder.Cells[priceTick] > D*_askLadder.Cells[priceTick+1] && _bidLadder.Cells[priceTick] > helper.diagonalThresholdVolume)returnData.CurrBrush = returnData.DiaStrongBidBrush;
				else returnData.CurrBrush = returnData.DiaNeutralBrush;
				
				returnData.FillColor = Color.FromArgb(50,Color.Gray);
				
				if(Math.Abs(volume) == Math.Abs(maxValue))returnData.CurrFont = returnData.DiaBoldFont;
				else returnData.CurrFont = returnData.DiaFont;
				
				returnData.CurrSF = farSF;
				return returnData;
				
			}
			
			DrawData DiagonalAskLadderFillParams(int volume,int maxValue,int priceTick)
			{
				DrawData returnData = BasicFillParams(volume,maxValue,helper.colorUp,priceTick);
				
				int N = helper.diaNumerator;
				int D = helper.diaDenominator;
				
				if(!_bidLadder.Cells.ContainsKey(priceTick-1)) returnData.CurrBrush= (_askLadder.Cells[priceTick] > helper.diagonalThresholdVolume) ? returnData.DiaStrongAskBrush : returnData.DiaNeutralBrush;
				else if(D*_askLadder.Cells[priceTick] < N*_bidLadder.Cells[priceTick-1] && _bidLadder.Cells[priceTick-1] > helper.diagonalThresholdVolume)returnData.CurrBrush = returnData.DiaWeakBrush;
				else if (N*_askLadder.Cells[priceTick] > D*_bidLadder.Cells[priceTick-1] && _askLadder.Cells[priceTick] > helper.diagonalThresholdVolume)returnData.CurrBrush = returnData.DiaStrongAskBrush;
				else returnData.CurrBrush = returnData.DiaNeutralBrush;
				
				returnData.FillColor = Color.FromArgb(50,Color.Gray);
				
				if(Math.Abs(volume) == Math.Abs(maxValue))returnData.CurrFont = returnData.DiaBoldFont;
				else returnData.CurrFont = returnData.DiaFont;
				
				returnData.CurrSF = nearSF;
				return returnData;
				
			}
			
			DrawData BidLadderFillParams(int volume,int maxValue,int priceTick)
			{
				int maxv;
				if(helper.ladderBATrueShape) maxv = Math.Max(Math.Abs(_bidLadder.MaxLadderValue),Math.Abs(_askLadder.MaxLadderValue));
				else maxv = maxValue;
					
				DrawData  returnData= BasicFillParams(volume,maxv,helper.colorDown,priceTick);
				
				if(helper.ladderBAHighSideMode)
				{
					if(!_bidLadder.Cells.ContainsKey(priceTick) || !_askLadder.Cells.ContainsKey(priceTick))return returnData;
					if(_bidLadder.Cells[priceTick] < _askLadder.Cells[priceTick])returnData.FillColor=helper.colorBackGround;
				}
				
				returnData.CurrBrush = returnData.BAFontBrush;
				if(helper.clt == CenterLadderType.BidXAsk) returnData.CurrSF = farSF;
				else returnData.CurrSF = centerSF;
				
				return returnData;
					
			}
				
			DrawData AskLadderFillParams(int volume,int maxValue, int priceTick)
			{
				
				int maxv;
				if(helper.ladderBATrueShape) maxv = Math.Max(Math.Abs(_bidLadder.MaxLadderValue),Math.Abs(_askLadder.MaxLadderValue));
				else maxv = maxValue;
					
				DrawData  returnData= BasicFillParams(volume,maxv,helper.colorUp,priceTick);
				
				if(helper.ladderBAHighSideMode)
				{
					if(!_bidLadder.Cells.ContainsKey(priceTick) || !_askLadder.Cells.ContainsKey(priceTick))return returnData;
					if(_bidLadder.Cells[priceTick] > _askLadder.Cells[priceTick])returnData.FillColor=helper.colorBackGround;
				}
				
				returnData.CurrBrush = returnData.BAFontBrush;
				if(helper.clt == CenterLadderType.BidXAsk) returnData.CurrSF = nearSF;
				else returnData.CurrSF = centerSF;
				
				return returnData;
					
				}	
				
			DrawData VolumeFillParams(int volume, int maxValue, int priceTick)
			{
				DrawData returnData=BasicFillParams(volume,maxValue,helper.colorCenterLadderVolume,priceTick);
				 
				returnData.CurrBrush = returnData.BAFontBrush;
				returnData.CurrSF = centerSF;
				return returnData;
			}
			
			DrawData CustomLadderFillParams(int volume, int maxValue, int priceTick)
			{
				DrawData returnData = new DrawData(helper);
				int delvol,maxdelvol;
				
				delvol = _deltaLadder.Cells[priceTick];
				maxdelvol = _deltaLadder.MaxLadderValue;
				float tempratio;
				float volratio;
				string bidString,askString;
				
				switch(helper.cls){
					case CustomLadderShape.Volume:
						returnData=BasicFillParams(volume,maxValue,helper.colorCenterLadderVolume,priceTick);
						returnData.CurrFont = returnData.BAFont;
						break;
					case CustomLadderShape.Delta:
						returnData=BasicFillParams((helper.ladderBANoMinus)?Math.Abs(delvol):delvol,maxdelvol,(delvol>0)?helper.colorUp:helper.colorDown,priceTick);
				        returnData.CurrFont = returnData.BAFont;
						break;
				}
				
				switch(helper.clb){
					case CustomLadderBase.Volume:
						volratio = (float)(_volumeLadder.Cells[priceTick])/(float)_volumeLadder.MaxLadderValue;
						returnData.FillColor = Color.FromArgb((int)(255*volratio),helper.colorCenterLadderVolume);
						break;
					case CustomLadderBase.Delta:
						tempratio=Math.Min(1f,(float)Math.Abs(delvol)/(float)Math.Max(1,Math.Abs(maxdelvol)));
						returnData.FillColor = (delvol > 0) ? Color.FromArgb((int)(255*tempratio),helper.colorUp) : Color.FromArgb((int)(255*tempratio),helper.colorDown);
						break;
				}
				
				switch(helper.cld){
					case CustomLadderData.Volume:
						returnData.FormattedData=LadderClass.Formatter(_volumeLadder.Cells[priceTick],helper.ladderBAFormat,helper);
						if(priceTick == _volumeLadder.MaxLadderValuePriceTick)returnData.CurrFont = returnData.BABoldFont;
						break;
					case CustomLadderData.Delta:
						returnData.FormattedData=LadderClass.Formatter((helper.clb == CustomLadderBase.Volume) ? delvol : Math.Abs(delvol),helper.ladderBAFormat,helper);
						if(priceTick == _deltaLadder.MaxLadderValuePriceTick)returnData.CurrFont = returnData.BABoldFont;
						break;
					case CustomLadderData.BidxAsk:
						bidString = LadderClass.Formatter(_bidLadder.Cells[priceTick],helper.ladderBAFormat,helper);
				        askString = LadderClass.Formatter(_askLadder.Cells[priceTick],helper.ladderBAFormat,helper);
						returnData.FormattedData = bidString + " X " + askString;
						if(priceTick == _volumeLadder.MaxLadderValuePriceTick)returnData.CurrFont = returnData.BABoldFont;
						break;
				}
				
				returnData.CurrBrush = returnData.BAFontBrush;
				returnData.CurrSF = centerSF;
				return returnData;
			}
			
				
			DrawData DeltaFillParams(int volume, int maxValue, int priceTick)
			{
				
				DrawData returnData=BasicFillParams((helper.ladderBANoMinus)?Math.Abs(volume):volume,maxValue,(volume>0)?helper.colorUp:helper.colorDown,priceTick);	
				
				returnData.CurrBrush = returnData.BAFontBrush;
				returnData.CurrSF = centerSF;
				return returnData;
			}
			
			
			#endregion
		}
		#endregion 
		
		#region GLADDER Class
		sealed class GLadderClass
		{
			//Fields & Properties
			
			//here is the collection of BidAskladders.
			Dictionary<int,LadderClass> _ladders = new Dictionary<int,LadderClass>();
			Dictionary<int,int> _cds=new Dictionary<int,int>();
			int _curBarNum;
			
			private LadderClass _curLadder=null;
			public LadderClass curLadder{get{ return _curLadder;}}
			
			private Helper _helper;
			private Helper helper{get {return _helper;}}
			
			protected FormattingType _ladderFormattingType;
			public FormattingType LadderFormattingType{get {return _ladderFormattingType;} set { _ladderFormattingType=value;}}
			
			protected RatioType _ladderRatioType;
			public RatioType LadderRatioType {get {return _ladderRatioType;} set { _ladderRatioType=value;}}
			
			protected int _ladderFixedNumber;
			public int LadderFixedNumber {get {return _ladderFixedNumber;} set { _ladderFixedNumber=value;}}
						
			protected LadderTypeEnum _ladderType;
			public LadderTypeEnum LadderType {get {return _ladderType;} set { _ladderType=value;}}
			
			protected AccumulationTypeEnum _accumulationType;
			public AccumulationTypeEnum AccumulationType {get {return _accumulationType;} set { _accumulationType=value;}}
			
			protected int _accumulationDays;
			public int AccumulationDays {get {return _accumulationDays;} set { _accumulationDays=value;}}
			
			protected CellFillType _gCellFillType= CellFillType.Histogram; // temp testing
			public CellFillType GCellFillType {get {return _gCellFillType;} set {_gCellFillType=value;}}
			
			protected Direction _gDirection= Direction.LeftToRight;   // temp testing
			public Direction GDirection {get {return _gDirection;} set {_gDirection=value;}}
			
			protected CellDataLocation _gDataLocation= CellDataLocation.Center;
			public CellDataLocation GDataLocation {get {return _gDataLocation;} set {_gDataLocation=value;}}
			
			protected SideLadderWidth _gWidth = SideLadderWidth.Medium;
			public SideLadderWidth GWidth {get {return _gWidth;} set {_gWidth=value;}}
			
			protected bool _gShowCellData = true;
			public bool GShowCellData {get {return _gShowCellData;} set {_gShowCellData=value;}}
			
			protected bool _show=false;
			public bool Show {get {return _show;} set { _show=value;}}		
						
						
			//Constructor
			
			public GLadderClass(LadderTypeEnum paramLadderType,AccumulationTypeEnum paramAccumulationType, 
								int paramAccumulationDays,FormattingType paramFormat, RatioType paramLadderRatioType,
				                int paramLadderFixedNumber, CellFillType paramCFT, bool paramShowCellData, Direction paramDirection,
				                CellDataLocation paramDataLocation, SideLadderWidth paramWidth)
			{
				_ladderType=paramLadderType;
				_accumulationType=paramAccumulationType;
				_accumulationDays=paramAccumulationDays;
				_ladderFormattingType=paramFormat;
				_ladderRatioType=paramLadderRatioType;
				_ladderFixedNumber=paramLadderFixedNumber;
				_gCellFillType=paramCFT;
				_gShowCellData=paramShowCellData;
				_gDirection=paramDirection;
				_gDataLocation=paramDataLocation;
				_gWidth=paramWidth;
			}
			
			
			public void AddLadder(int barNum,bool newSession,int endedsession,Helper paramHelper)
			{ 
				if (!paramHelper.liveMode || newSession || (_curLadder==null))
				{
				LadderClass initLadder=new LadderClass(null); 
				
		    {
					
				if ((!newSession) || (_accumulationType==AccumulationTypeEnum.AllDays))
				{	
					_ladders.Add(barNum,new LadderClass(curLadder,paramHelper));
					if (_cds.ContainsKey(barNum-1))
                        _cds.Add(barNum,_cds[barNum-1]);
                    else
                        _cds.Add(barNum, 0);
				}
				else if (_accumulationType==AccumulationTypeEnum.ZeroDay)
				{
					_ladders.Add(barNum,new LadderClass(paramHelper));
					_cds.Add(barNum,0);
				}
				else
					{	_cds.Add(barNum,0);
						
						for(int i=_accumulationDays-1;i>=0;i--)
						{	
							if(_ladderType==LadderTypeEnum.Bid)
							{
								if (helper.dailyBidLadders.ContainsKey(endedsession-i))
										initLadder.Add(helper.dailyBidLadders[endedsession-i]);
							}
							else if (_ladderType==LadderTypeEnum.Ask)
							{
								if (helper.dailyAskLadders.ContainsKey(endedsession-i))
										initLadder.Add(helper.dailyAskLadders[endedsession-i]);
							}
							else if (_ladderType==LadderTypeEnum.Delta)
								{
									if (helper.dailyDeltaLadders.ContainsKey(endedsession-i))
										initLadder.Add(helper.dailyDeltaLadders[endedsession-i]);
								}
							else if (_ladderType==LadderTypeEnum.TotalVolume)
								{	
									if (helper.dailyVolLadders.ContainsKey(endedsession-i))
										initLadder.Add(helper.dailyVolLadders[endedsession-i]);
								}
							else
								{
									if (helper.dailyDeltaDiffLadders.ContainsKey(endedsession-i))
									{	
										initLadder.CopyWithOffset(helper.dailyDeltaDiffLadders[endedsession-i],_cds[barNum],true);
										_cds[barNum]+= helper.dailyCDs[endedsession-i];
										
									}
								}
									
						}
						_ladders.Add(barNum,new LadderClass(initLadder,paramHelper));
					}
				}
				
				_curLadder=_ladders[barNum];
				_curLadder.FillCells=TotFillParams;
				_helper=paramHelper;
				_curBarNum=barNum;
				}
				
			}
				
			public void Add(int priceTick, int volume)
			{		
				_curLadder.Increment(priceTick,volume,false);

			}
			
			public void SetCD(int priceTick, int volume)
			{		
				_cds[_curBarNum]+=volume;
				_curLadder.Set(priceTick,_cds[_curBarNum]);
			}
			
			//Grahical methods
			public void DrawLadder(int barNum,float x)
			{	
				LadderClass ladderToDraw=null;
				LadderClass DiffLadder;
				int bNum=barNum;
				
				if (_ladders.ContainsKey(barNum))
					ladderToDraw=_ladders[barNum];				
				else 
				{	
					ladderToDraw=_curLadder;	
					bNum=_curBarNum;
				}
				
				if (ladderToDraw != null)
				{	
					if (_ladderType==LadderTypeEnum.DeltaDiff)
					{
						DiffLadder=new LadderClass(_helper);
						DiffLadder.FillCells=TotDeltaFillParams;
						DiffLadder.CopyWithOffset(ladderToDraw,_cds[bNum],false);
						ladderToDraw=DiffLadder;
					}
					ladderToDraw.ComputeMax();
					ladderToDraw.EmptyLadder(x,(float)_gWidth);
					
					ladderToDraw.DrawGeneralLadderFillCells(x,(float)_gWidth,_gDirection,helper.showBordersSide,true,helper.showNumbersSide && _gShowCellData);
					ladderToDraw.DrawCell(x,(float)_gWidth,ladderToDraw.MaxLadderValuePriceTick,helper.myVolPen);
				}
			}
			
			
			//2 Ladder Custom Fill Delegates	
			DrawData GCommonFillParam(int volume, int maxValue, ref float ratio, int priceTick)
			{
				DrawData returnData=new DrawData(helper);
				
				if (maxValue==0) 
					ratio=0;
				else
				{	
					if (_ladderRatioType==RatioType.MaximumOfLadder)
						ratio=(float)Math.Abs(volume)/(float)Math.Abs(maxValue);
					else
						ratio=Math.Min(1f,(float)Math.Abs(volume)/(float)_ladderFixedNumber);	
				}
				if(_gCellFillType == CellFillType.Histogram)returnData.FillEndRatio = ratio;
				else if(_gCellFillType == CellFillType.Full)returnData.FillEndRatio = 1;
				else if(_gCellFillType == CellFillType.None)returnData.FillEndRatio = 0;
				
				returnData.FormattedData=LadderClass.Formatter(volume,_ladderFormattingType,helper);
				
				if(Math.Abs(volume) == Math.Abs(maxValue))returnData.CurrFont = returnData.SideBoldFont;
				else returnData.CurrFont = returnData.SideFont;
				returnData.CurrBrush = returnData.SideFontBrush;
				
				if(_gDataLocation == CellDataLocation.Right)returnData.CurrSF = farSF;
				else if(_gDataLocation == CellDataLocation.Left)returnData.CurrSF = nearSF;
				else returnData.CurrSF = centerSF;
				
				return returnData;
			}
			
			DrawData TotBidFillParams(int volume, int maxValue, int priceTick)
			{
				DrawData returnData=new DrawData(helper);
				float ratio=0.0f;
				
				returnData = GCommonFillParam(volume,maxValue,ref ratio,priceTick);
				returnData.FillColor=Color.FromArgb((int)(255*ratio),helper.colorDown);
				
				return returnData;
			}
			
			DrawData TotAskFillParams(int volume, int maxValue, int priceTick)
			{
				DrawData returnData=new DrawData(helper);
				float ratio=0.0f;
				
				returnData = GCommonFillParam(volume,maxValue,ref ratio,priceTick);
				returnData.FillColor=Color.FromArgb((int)(255*ratio),helper.colorUp);
				
				return returnData;
			}
			
			DrawData TotDeltaFillParams(int volume, int maxValue, int priceTick)
			{
				DrawData returnData=new DrawData(helper);
				float ratio=0.0f;
				
				returnData = GCommonFillParam(volume,maxValue,ref ratio,priceTick);
				returnData.FillColor=Color.FromArgb((int)(255f*ratio),(volume>0)?helper.colorUp:helper.colorDown);
				
				return returnData;
			}
			
		
			DrawData TotVolFillParams(int volume, int maxValue, int priceTick)
			{
				DrawData returnData=new DrawData(helper);
				float ratio=0.0f;
				
				returnData = GCommonFillParam(volume,maxValue,ref ratio,priceTick);
				returnData.FillColor=Color.FromArgb((int)(255*ratio),helper.colorSideLadderVolume);
				
				return returnData;
			}
			
			
			DrawData TotFillParams(int volume, int maxValue, int priceTick)
			{
				if(_ladderType==LadderTypeEnum.Bid)   return TotBidFillParams(volume,maxValue,priceTick);
				else if(_ladderType==LadderTypeEnum.Ask)   return TotAskFillParams(volume,maxValue,priceTick);
				else if (_ladderType==LadderTypeEnum.Delta) 	return TotDeltaFillParams(volume,maxValue,priceTick);
				else 									        return TotVolFillParams(volume,maxValue,priceTick);
			}
			
		}
		#endregion
		
		#region Helper Class
		class Helper
		{
			
			public float scalingfactor;
			public float graphoffset;
			public float y0;
			public Graphics graphics;
			public ChartControl chartcontrol;
			public float BAcellwidth;
			public Rectangle bounds;
			public double currentmin=0f,currentmax=0f;
			public Font baFont;
			public Font sideFont;
			public Font bottomFont;
			public Font diaFont;
			public Color baFontColor=Color.Black;
			public Color sideFontColor = Color.Black;
			public Color bottomFontColor = Color.Blue;
			public Color colorUp=Color.Green;
			public Color colorDown=Color.Red;
			public Color colorSideLadderVolume=Color.DodgerBlue;
			public Color colorCenterLadderVolume=Color.DodgerBlue;
			public Color colorBackGround=Color.Transparent;
			public Color legendBackColor = Color.DodgerBlue;
			public Color DiaStrongAskColor = Color.LimeGreen;
			public Color DiaStrongBidColor = Color.IndianRed;
			public Color DiaWeakColor = Color.DarkSlateBlue;
			public Color DiaNeutralColor = Color.Gray;
			public SolidBrush myWhiteBrush;
			public Pen myOpenPen;						
			public Pen myTickPen;
			public Pen myVolPen;
			public Pen myVol2Pen;
			public Pen myBAborderPen;
			public Pen myVwapPen;
			public FormattingType ladderBAFormat=FormattingType.Normal;
			public int lotSize = 50;
			public int ladderBAFixedNumber=15000;
			public int contractReversalSetting=3000;
		    public int minTick,maxTick;	
			public double BALadderHistogramSpacing=.85;
			public double SideLadderHistogramSpacing=1;
			public int CumulativeDelta=0;
			public bool abrev=true;
			public bool BlackBackground;
			public bool	firstTickOfBar = true;			// When it's the first tick of a new bar, we need to update the open price.			
			public bool showBALadder=true;
			public bool ShowERLadder=false;
			public bool liveMode=false;
			public bool BAdeltaMode=false;
			public bool SideLadderFlag=false;
			public bool showhotkeys=false;
			public bool showBordersBA=true;
			public bool showNumbersSide=true;
			public bool showBordersSide=true;
			public bool showNumbersDeltaOnly=true;
			public bool ladderBATrueShape=false;
			public bool ladderBAShowMaxLevel=true;	
			public bool ladderBAShowCOT=false;	
			public bool ladderBAShowOuterBoarder=false;
			public bool ladderBAHighSideMode=false;
			public int  ladderBAHighSidePercent=30;
			public bool ladderBAShowNumbers=false;
			public bool ladderBAShowBidAskOnDeltaBase=false;
				
			public bool ladderBANoMinus=true;
			public bool ladderBAShowUpReversal=false;
			public bool ladderBAShowDwnReversal=false;
			public bool ladderBAShowMaxDeltaView=true;
			
			public bool showVWAP=false;
			public int agregationFib=1;
			public int agregationFactor;
			
			public Dictionary<int,LadderClass> dailyBidLadders = new Dictionary<int,LadderClass>();
			public Dictionary<int,LadderClass> dailyAskLadders = new Dictionary<int,LadderClass>();
			public Dictionary<int,LadderClass> dailyVolLadders = new Dictionary<int,LadderClass>();
			public Dictionary<int,LadderClass> dailyDeltaLadders = new Dictionary<int,LadderClass>();
			public Dictionary<int,LadderClass> dailyDeltaDiffLadders= new Dictionary<int,LadderClass>();
			
			public Dictionary<int,int> dailyCDs= new Dictionary<int,int>();
			public Bars bars;
			public IntSeries dsHighCOT,dsLowCOT;
			public IntSeries dsMaxBid,dsMaxAsk,dsSumBid,dsSumAsk,dsDelta,dsVolume;
			public IntSeries dsSessionCD,dsSessionCV,dsCumCD; 
			public IntSeries dsSessionVWAP;   
			
			public RowNumber volRow,deltaRow,cumDeltaRow,cumVolRow,minDeltaRow,maxDeltaRow,sellerRow,buyerRow;  
			public RowNumber pctDeltaRow,pctCumDeltaRow,pctSellerRow,pctBuyerRow,pctMinDeltaRow,pctMaxDeltaRow;
			public RowNumber finishRow,startRow;
			public Dictionary<string,int> legends = new Dictionary<string,int>();
			
			public CenterLadderType clt = CenterLadderType.BidAndAsk;
			public CellColorScheme ccs = CellColorScheme.MaximumOfLadder;
			public CellFillType cftCenter = CellFillType.Histogram;
			
			public int extraRightSpace = 8;
			public LegendLocation bottomLL = LegendLocation.Right;
			public BottomViewType bottomview = BottomViewType.None;
			public float bottomGraphicHeight = 150;
			public BottomGraphicHeight bgh = BottomGraphicHeight.Medium;
			public BottomGraphicType bottomgraphic = BottomGraphicType.Volume;
			public Color colorBottomGraphicBase = Color.DodgerBlue;
			public int bgBaseOpacity = 50;
			public Color colorBottomGraphicVol = Color.SlateBlue;
			public Color colorBottomGraphicBid = Color.Crimson;
			public Color colorBottomGraphicAsk = Color.SeaGreen;
			public Pen myBGLinePen = new Pen(Color.Black,2);
			public Pen myBGLinePen1 = new Pen(Color.Black,1);
			public int diagonalThresholdVolume = 10000;
			public int diaNumerator = 7;
			public int diaDenominator = 10;
			
			public Color customCandleUpColor = Color.LawnGreen;
			public Color customCandleDnColor = Color.Red;
			public Color customCandleNeutralColor = Color.Silver;
			public int customCandleThickness = 2;
			public int customCandleFillOpacity = 5;
			
			public CustomLadderBase clb = CustomLadderBase.Volume;
			public CustomLadderShape cls = CustomLadderShape.Volume;
			public CustomLadderData cld = CustomLadderData.Volume;
			public Direction clDirection = Direction.LeftToRight;
			public Direction candleDirection = Direction.LeftToRight;
			
			public Dictionary<int,int> FirstBarIdxOfSession = new Dictionary<int,int>();
			public Helper()
			{
				myWhiteBrush =new SolidBrush(colorBackGround);
				myTickPen=new Pen(Color.Orange,2);		// sets color and line thickness of last tick
				myOpenPen=new Pen(Color.Crimson,2);		// sets color and line thickness of first tick (open of bar)
				myVolPen=new Pen(Color.Blue,2);	// sets color and line thinkness of max volume
				myVol2Pen =new Pen(Color.OrangeRed,2);
				myVwapPen = new Pen(Color.Red,2);
				myBAborderPen=new Pen(Color.Black,1);
				agregationFactor=Fibonacci(agregationFib);
				baFont = new Font("Arial",8,FontStyle.Regular);
				sideFont = new Font("Arial",8,FontStyle.Regular);
				bottomFont = new Font("Arial",9,FontStyle.Bold);
				diaFont = new Font("Arial",8,FontStyle.Regular);
				
			}
			
			public float GetY(float ticklevel)
			{
					return(graphoffset - (ticklevel+((float)agregationFactor-1)/(2*(float)agregationFactor)- y0) * scalingfactor);
			}		
		}
		#endregion

		#region BA Ladder Collection & Miss
		//here is the collection of BidAskladders.
		Dictionary<int,BALadderClass> baLadders = new Dictionary<int,BALadderClass>();
		
		BALadderClass curBALadder=null;
		
		
		LadderClass curDailyBidLadder,curDailyAskLadder,curDailyVolLadder,curDailyDeltaLadder,curDailyDeltaDiffLadder;

		Dictionary<string,GLadderClass> gLadders=new Dictionary<string,GLadderClass>();

		GLadderClass gLadderExtremeLeft= new GLadderClass(LadderTypeEnum.TotalVolume,AccumulationTypeEnum.AllDays,0,
										FormattingType.Kilo,RatioType.MaximumOfLadder,5000,CellFillType.Histogram,true,Direction.RightToLeft,
			                             CellDataLocation.Center,SideLadderWidth.Medium);
			
		GLadderClass gLadderLeft=new GLadderClass(LadderTypeEnum.TotalVolume,AccumulationTypeEnum.ZeroDay,0,
										FormattingType.Kilo,RatioType.MaximumOfLadder,5000,CellFillType.Full,true,Direction.LeftToRight,
			                            CellDataLocation.Center,SideLadderWidth.Medium);
		
		GLadderClass gLadderRight=new GLadderClass(LadderTypeEnum.DeltaDiff,AccumulationTypeEnum.ZeroDay,0,
										FormattingType.Normal,RatioType.MaximumOfLadder,5000,CellFillType.Full,true,Direction.RightToLeft,
			                            CellDataLocation.Center,SideLadderWidth.Medium);
		
		GLadderClass gLadderExtremeRight=new GLadderClass(LadderTypeEnum.DeltaDiff,AccumulationTypeEnum.AllDays,0,
										FormattingType.Normal,RatioType.MaximumOfLadder,25000,CellFillType.Histogram,true,Direction.LeftToRight,
			                             CellDataLocation.Center,SideLadderWidth.Medium);
		
		
		
		Helper helper=new Helper();

		int lastBar=-1;
		int curSession=0;
		int curDailyCD=0;
		int curCumV=0;
		int cumCD = 0;
		double curPV=0;
		double askPrice,bidPrice;
		
		static StringFormat centerSF=new StringFormat();
		static StringFormat farSF = new StringFormat();
		static StringFormat nearSF = new StringFormat();
		bool firstinitdone=false;
		bool forcegoodsize=true;
		

		System.Windows.Forms.KeyEventHandler keyEvtH;
		
		double initCellHeight;
		private int initBarSpace=80;
		private int initBarWidth=2;
		
				
		static SolidBrush myBlackBrush=new SolidBrush(Color.Black);
		static Color cup,cdn,ccandle;
		
		static Pen myBlackPen=new Pen(Color.Black);// changes out line color
		static Pen legendPen = new Pen(Color.Black,2);

		static int Fibonacci (int x)
		{

   			if (x <= 1)
   			{
      			return 1;
   			}
   		return Fibonacci (x-1) + Fibonacci (x-2);
		}		
		
		#endregion
		
		#region GomInitialize
        protected override void GomInitialize()
        {		
            try{
				
			CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
			AutoScale 			= true;

			centerSF.Alignment=StringAlignment.Center;
			centerSF.LineAlignment=StringAlignment.Center;
				
			farSF.Alignment = StringAlignment.Far;
			farSF.LineAlignment = StringAlignment.Center;
			
			nearSF.Alignment = StringAlignment.Near;
			nearSF.LineAlignment = StringAlignment.Center;
			
			helper.dailyBidLadders.Add(0,new LadderClass(null));
			curDailyBidLadder=helper.dailyBidLadders[0];
				
			helper.dailyAskLadders.Add(0,new LadderClass(null));
			curDailyAskLadder=helper.dailyAskLadders[0];
				
			helper.dailyVolLadders.Add(0,new LadderClass(null));
			curDailyVolLadder=helper.dailyVolLadders[0];
			
			helper.dailyDeltaLadders.Add(0,new LadderClass(null));
			curDailyDeltaLadder=helper.dailyDeltaLadders[0];
			
			helper.dailyDeltaDiffLadders.Add(0,new LadderClass(null));
			curDailyDeltaDiffLadder=helper.dailyDeltaDiffLadders[0];

			helper.dailyCDs.Add(0,0);
			
			gLadders.Add("EL",gLadderExtremeLeft);
			gLadders.Add("L",gLadderLeft);
			gLadders.Add("R",gLadderRight);
			gLadders.Add("ER",gLadderExtremeRight);
			
			GC.Collect();
      		GC.WaitForPendingFinalizers();
			
			helper.dsHighCOT = new IntSeries(this,MaximumBarsLookBack.Infinite);
			helper.dsLowCOT = new IntSeries(this,MaximumBarsLookBack.Infinite);
			helper.dsMaxBid = new IntSeries(this,MaximumBarsLookBack.Infinite);
			helper.dsMaxAsk = new IntSeries(this,MaximumBarsLookBack.Infinite);
			helper.dsSumBid = new IntSeries(this,MaximumBarsLookBack.Infinite);
			helper.dsSumAsk = new IntSeries(this,MaximumBarsLookBack.Infinite);
			helper.dsVolume = new IntSeries(this,MaximumBarsLookBack.Infinite);
			helper.dsDelta = new IntSeries(this,MaximumBarsLookBack.Infinite);
			helper.dsSessionCD = new IntSeries(this,MaximumBarsLookBack.Infinite); 
			helper.dsSessionCV = new IntSeries(this,MaximumBarsLookBack.Infinite); 
		    helper.dsCumCD = new IntSeries(this,MaximumBarsLookBack.Infinite); 
		    helper.dsSessionVWAP = new IntSeries(this,MaximumBarsLookBack.Infinite);
				
			
				
			} catch (Exception ex){
			Print("GomLadder Initialize: " + ex.Message);
				return;
			}	
		
		}
        #endregion
		
		#region GomOnStartUp
		protected override void GomOnStartUp()
		{
			cup = this.ChartControl.ChartStyle.UpColor;
			cdn = this.ChartControl.ChartStyle.DownColor;
			ccandle = this.CandleOutlineColor;
			
			keyEvtH=new System.Windows.Forms.KeyEventHandler(this.chart_KeyDown);	
			this.ChartControl.ChartPanel.KeyDown += keyEvtH;
			
		}
		#endregion
		
		#region HotKeys
		private void chart_KeyDown (object sender, KeyEventArgs e)// coding for quick keys
		{
			if(e.Control && e.KeyCode==Keys.F4)
			{
				if(helper.ccs == CellColorScheme.DominateSide) helper.ccs = CellColorScheme.MaximumOfLadder;
				else helper.ccs++;
			}
			
			
			else if (e.KeyCode==Keys.Multiply)
			{
				if(helper.clt == CenterLadderType.DiagonalDelta)helper.clt = 0;
				else helper.clt++;
			}
			
			else if (e.KeyCode==Keys.Divide)
			{
				if(helper.clt == CenterLadderType.BidAndAsk)helper.clt = CenterLadderType.DiagonalDelta;
				else helper.clt--;
			}
			
			else if (e.KeyCode==Keys.Add)
			{	
				helper.agregationFib++;
				SendKeys.Send("{F5}");
			}
			else if (e.KeyCode==Keys.Subtract)
			{ 	
				if (helper.agregationFactor>1)					
					{
						helper.agregationFib--;
						SendKeys.Send("{F5}");
					}
			}
			else if (e.KeyCode==Keys.Up)
			{ 	
				double delta;	
				delta=this.TickSize*helper.agregationFactor;
				helper.currentmax -= delta;
				helper.currentmin -= delta;
			}
			else if (e.KeyCode==Keys.Down)
			{ 	
				double delta;	
				delta=this.TickSize*helper.agregationFactor;
				helper.currentmax += delta;
				helper.currentmin += delta;
			}
			
			else if (e.KeyCode==Keys.Space)
			{ 	
				if((int)helper.bottomgraphic == 2) helper.bottomgraphic = 0;
				else helper.bottomgraphic++;
			}
			
			else if (e.KeyCode==Keys.Decimal)
			{ 	
				
				if(helper.cftCenter == CellFillType.Full)helper.cftCenter = CellFillType.None;
				else helper.cftCenter++;
			}
			
			else if (e.Shift&&e.KeyCode==Keys.F4)// Hot Key Help
			{ 	
				helper.showhotkeys=!helper.showhotkeys;
			}

			else if (e.Shift&&e.KeyCode==Keys.F7)// numbers on side
			{ 	
				helper.showNumbersSide=!helper.showNumbersSide;
			}
			
			else if (e.Shift&&e.KeyCode==Keys.F6)// numbers BA
			{ 	
				helper.ladderBAShowNumbers=!helper.ladderBAShowNumbers;
		   }
			
			
			else if (e.Shift&&e.KeyCode==Keys.F8)
			{ 	
				helper.showBordersSide=!helper.showBordersSide;
				
			}
			
			else if (e.Shift&&e.KeyCode==Keys.F5)
			{ 	
				helper.showBordersBA=!helper.showBordersBA;
				
			}
			
			
			else if (e.Shift&&e.KeyCode==Keys.F9)// bottom view circulate
			{ 	
				if((int)helper.bottomview == 2) helper.bottomview = 0;
				else helper.bottomview++;
			}
			
						
			else if (e.Alt&&e.KeyCode==Keys.F12)
			{ 	
				helper.abrev=!helper.abrev;
			}
			
			else if (e.Alt&&e.KeyCode==Keys.F10)
			{ 	
				helper.ladderBAShowCOT=!helper.ladderBAShowCOT;
			}
			
			
			else if (e.Alt&&e.KeyCode==Keys.F11)
			{ 	
			helper.ladderBAShowDwnReversal=!helper.ladderBAShowDwnReversal;
			helper.ladderBAShowUpReversal=!helper.ladderBAShowUpReversal;
			}
			
			else if(e.Shift&&e.KeyCode==Keys.F2)
			{
				helper.ladderBAHighSideMode=!helper.ladderBAHighSideMode;
			}
			
			else if(e.Shift&&e.KeyCode==Keys.F12)
			{
				helper.ladderBATrueShape=!helper.ladderBATrueShape;
			}
			
		}
		#endregion
		
		#region GomOnBarUpdate
		protected override void GomOnBarUpdate()
			
		{
			try{
				
			if (CurrentBar != lastBar)
			{
				this.CandleOutlineColor = Color.Transparent;
				lastBar=CurrentBar;
				helper.dsHighCOT.Set(0);
				helper.dsLowCOT.Set(0);
				helper.dsMaxBid.Set(0);
				helper.dsMaxAsk.Set(0);
				
				helper.dsSumBid.Set(0);
				helper.dsSumAsk.Set(0);
				helper.dsDelta.Set(0);
				helper.dsVolume.Set(0);
				helper.dsSessionCD.Set(0);
				helper.dsSessionCV.Set(0);
				
				helper.dsCumCD.Set(cumCD);
				
				//add new ladders
				if (Bars.SessionBreak)
				{
					
					helper.dailyCDs[curSession]=curDailyCD;
					curDailyCD=0;
					curSession++;	
					curCumV=0;  
					curPV =0;
					
					helper.dailyBidLadders.Add(curSession,new LadderClass(null));
					helper.dailyAskLadders.Add(curSession,new LadderClass(null));
					helper.dailyVolLadders.Add(curSession,new LadderClass(null));
					helper.dailyDeltaLadders.Add(curSession,new LadderClass(null));
					helper.dailyDeltaDiffLadders.Add(curSession,new LadderClass(null));	
					helper.dailyCDs.Add(curSession,0);
					
					curDailyBidLadder = helper.dailyBidLadders[curSession];
					curDailyAskLadder = helper.dailyAskLadders[curSession];
					curDailyVolLadder = helper.dailyVolLadders[curSession];
                    curDailyDeltaLadder = helper.dailyDeltaLadders[curSession];
                    curDailyDeltaDiffLadder = helper.dailyDeltaDiffLadders[curSession];
					
					helper.FirstBarIdxOfSession.Add(curSession,CurrentBar);
					
				}
				
				
				foreach (GLadderClass ladder in gLadders.Values)
				{
					if (ladder.Show)
						ladder.AddLadder(CurrentBar,Bars.SessionBreak,curSession-1,helper);
				}
				
				baLadders.Add(CurrentBar,new BALadderClass(CurrentBar,helper));
				curBALadder=baLadders[CurrentBar];

			}
			
			
		} catch (Exception ex){
			Print("GomLadder OnBarUpdate: " + ex.Message);
				return;
			}	
		}
		#endregion
		
		#region GomOnMarketData
		protected override void GomOnMarketData(TickTypeEnum tickType,double price,int volume)
        {	
			try{
			int priceTick; int deltavol=0,bidvol=0,askvol=0;
					
			priceTick=Convert.ToInt32(price/this.TickSize)/helper.agregationFactor;
			
			deltavol=CalcDelta(tickType,price,volume);
			
			if (deltavol>0) {curBALadder.AddBidAskVolume(priceTick,0,volume);bidvol=0;askvol=volume;}
			if (deltavol<0) {curBALadder.AddBidAskVolume(priceTick,volume,0);bidvol=volume;askvol=0;}
			if (deltavol==0)return;
			
			{ 
				helper.dsSumAsk.Set(curBALadder.AskLadder.Sum);
				helper.dsSumBid.Set(curBALadder.BidLadder.Sum);
			
				if (curBALadder.AskLadder.MaxLadderValue != Int32.MinValue )
					helper.dsMaxAsk.Set(curBALadder.AskLadder.MaxLadderValue);
				if (curBALadder.BidLadder.MaxLadderValue != Int32.MinValue )
					helper.dsMaxBid.Set(curBALadder.BidLadder.MaxLadderValue);
				helper.dsDelta.Set(curBALadder.AskLadder.Sum-curBALadder.BidLadder.Sum);
				helper.dsVolume.Set(curBALadder.AskLadder.Sum+curBALadder.BidLadder.Sum);
				
				curBALadder.CalcLevelsBA(priceTick);
				helper.dsHighCOT.Set(curBALadder.HighCOT);
				helper.dsLowCOT.Set(curBALadder.LowCOT);
			}
			
			if ((volume !=0))
			{
				foreach (GLadderClass ladder in gLadders.Values)
				{
					if (ladder.Show) 
					{
						if (ladder.LadderType == LadderTypeEnum.Bid)
						    ladder.Add(priceTick,bidvol);
						else if (ladder.LadderType == LadderTypeEnum.Ask)
							ladder.Add(priceTick,askvol);
						else if (ladder.LadderType==LadderTypeEnum.Delta)
							ladder.Add(priceTick,deltavol);
						else if (ladder.LadderType==LadderTypeEnum.TotalVolume)
						    ladder.Add(priceTick,volume);
						else 
							ladder.SetCD(priceTick,deltavol);
					}
				}
				
			helper.dailyCDs[curSession]+=deltavol;
				
			curDailyCD+=deltavol;
			cumCD += deltavol;
			curCumV +=Math.Abs(deltavol);
			helper.dsSessionCD.Set(curDailyCD);
			helper.dsSessionCV.Set(curCumV);
			helper.dsCumCD.Set(cumCD);
				
			if(helper.showVWAP)
			{
			int vTick;	
			curPV  += price*volume;
				
			vTick = Convert.ToInt32((curPV/curCumV)/this.TickSize)/helper.agregationFactor;
			helper.dsSessionVWAP.Set(vTick);
			}
			
			curDailyBidLadder.Increment(priceTick,bidvol,false);
			curDailyAskLadder.Increment(priceTick,askvol,false);
			curDailyDeltaLadder.Increment(priceTick,deltavol,false);	
			curDailyVolLadder.Increment(priceTick,volume,false);
			curDailyDeltaDiffLadder.Set(priceTick,curDailyCD);
			}
			} catch (Exception ex){
			Print("GomLadder OnMarketData: " + ex.Message);
				return;
			}			
		}
		#endregion
		
		#region GetMinMax
		public override void GetMinMaxValues(ChartControl chartControl, ref double min, ref double max)
		{
			try{
				if (Bars==null)	return;
			
			int lastBar		= Math.Min(ChartControl.LastBarPainted, Bars.Count - 1);
			int firstBar	= Math.Max((ChartControl.LastBarPainted - ChartControl.BarsPainted) + 1,0);
			
			double newmax, newmin;
			double excursion;
			initCellHeight = helper.baFont.GetHeight();		
			newmax=((((int)(Bars.Get(lastBar).High/this.TickSize))/helper.agregationFactor+2)*helper.agregationFactor)*this.TickSize;
			newmin=((((int)(Bars.Get(lastBar).Low/this.TickSize))/helper.agregationFactor-1)*helper.agregationFactor-2)*this.TickSize;
			excursion=(Math.Floor(helper.bounds.Height/initCellHeight)-1)*helper.agregationFactor*this.TickSize;
			
			if (helper.bounds.Height>0)
			{	
				if (helper.currentmax<1) //init 
				{
					helper.currentmax=(((int)(((newmax+newmin+excursion)/2.0)/this.TickSize))/helper.agregationFactor)*helper.agregationFactor*this.TickSize;
					helper.currentmin=helper.currentmax-excursion;
	
				}
				else if (newmax>helper.currentmax)
				{
					helper.currentmax=newmax;
					helper.currentmin=newmax-excursion;
				}
				else if (newmin<helper.currentmin)
				{
					helper.currentmin=newmin;
					helper.currentmax=newmin+excursion;
				}			
	
				min=helper.currentmin;
				max=helper.currentmax;
			}
			} catch (Exception ex){
			Print("GomLadder GetMinMaxValues: " + ex.Message);
				return;
			}	
			
		}
		#endregion	
			
		#region Plot		
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			base.Plot(graphics,bounds,min,max);
			try{
			if (Bars==null)	return;
           
			int lastBar		= Math.Min(this.LastBarIndexPainted, Bars.Count - 1);
			int firstBar	= this.FirstBarIndexPainted;
			
			
			helper.agregationFactor=Fibonacci(helper.agregationFib);
			helper.scalingfactor=(float)(bounds.Height/ChartControl.MaxMinusMin(max, min)*this.TickSize)*helper.agregationFactor;
			// scalingfactor is equivalent to height of 1 Cell.
			// agregationfactor is equivalent to number of ticks per cell.
			helper.y0=(float)(((float)(min/this.TickSize)))/helper.agregationFactor;
			helper.graphoffset=(float)(bounds.Y + bounds.Height);
			helper.graphics=graphics;
			helper.chartcontrol=base.ChartControl;
			
			helper.minTick=((int)(min/this.TickSize))/helper.agregationFactor;
			helper.maxTick=((int)(max/this.TickSize))/helper.agregationFactor;			
			helper.bounds=bounds;
			helper.bars=BarsArray[0];
			
			helper.BAcellwidth=InitBarSpace/2.0f-2*InitBarWidth-1;
			
			int totalRightSideSpace = ((gLadderRight.Show) ? (int)gLadderRight.GWidth : 0) +
				                 ((gLadderExtremeRight.Show) ? (int)gLadderExtremeRight.GWidth : 0) +
				                  helper.extraRightSpace;
			
			if 	(ChartControl.BarMarginRight!=(int)helper.BAcellwidth+totalRightSideSpace)
				ChartControl.BarMarginRight=(int)helper.BAcellwidth+totalRightSideSpace;
					
			if (!firstinitdone)
			{	
				firstinitdone=true;
				this.ChartControl.ChartStyle.UpColor = Color.Transparent;
				this.ChartControl.ChartStyle.DownColor = Color.Transparent;
			}
			
			InitBarSpace = helper.chartcontrol.BarSpace;
			InitBarWidth = helper.chartcontrol.BarWidth;
			
			float xleft,xright;
			xleft=bounds.Left ;
			xright=ChartControl.GetXByBarIdx(BarsArray[0],ChartControl.LastBarPainted)+ChartControl.BarWidth+helper.BAcellwidth+3;
			
			for (int index = lastBar; index >= firstBar; index--)
			{
				if (baLadders.ContainsKey(index))
					baLadders[index].DrawLadder();
			}	
			
			foreach (KeyValuePair<string,GLadderClass> ladderIterator in gLadders)
				{   helper.SideLadderFlag=true;
					if (ladderIterator.Value.Show) 
					{
						if (string.Compare(ladderIterator.Key,"EL")==0)
							ladderIterator.Value.DrawLadder(lastBar,xleft);
						else if (string.Compare(ladderIterator.Key,"L")==0)
							ladderIterator.Value.DrawLadder(lastBar,2f+xleft+(float)gLadderExtremeLeft.GWidth);
						else if (string.Compare(ladderIterator.Key,"R")==0)
							ladderIterator.Value.DrawLadder(lastBar,xright);
						else if (string.Compare(ladderIterator.Key,"ER")==0)
							ladderIterator.Value.DrawLadder(lastBar,2f+xright+((gLadderRight.Show)? (float)gLadderRight.GWidth:0));
					}
				}	
			
			if(helper.bottomview == BottomViewType.Statistic && helper.bottomLL != LegendLocation.None)
			{
				
				float w = ((gLadderLeft.Show) ? (float)gLadderLeft.GWidth : 0) +
					       ((gLadderExtremeLeft.Show) ? (float)gLadderExtremeLeft.GWidth : 0);
				w = Math.Max(80,w);
			foreach (KeyValuePair<string,int> legenditer in helper.legends)
			{
				if(helper.bottomLL == LegendLocation.Right)
					DrawBottomLegend(xright+4,helper.bounds.Right - xright-8,helper,legenditer.Key,legenditer.Value);
				else DrawBottomLegend(xleft+2,w,helper,legenditer.Key,legenditer.Value);
			}
			}
			
			if(helper.bottomview == BottomViewType.Graphic)
				DrawBottomGraphicView(helper,firstBar,lastBar,baLadders);
			
			Color ColorNeutral=Color.FromArgb(255,ChartControl.BackColor.R,ChartControl.BackColor.G,ChartControl.BackColor.B);
			
			string msg="";
			if(Bars.PercentComplete!=null&&Bars.PercentComplete!=0)  
			msg = msg+"  Current bar completion: "+(int)(100*Bars.PercentComplete)+"%";
			msg=msg+"\r\n  "+recordingMessage+" - Price Aggreg : "+helper.agregationFactor;
			
				msg= msg+"\r\n \r\n  List of Hotkey Codes";
				msg=msg+"\r\n  ********************************* \r\n  Shift +F4: Show/Hide This List of Hotkey Codes";
				msg=msg+"\r\n  Space Bar: Circulate Bottom Graphic Type";
				msg=msg+"\r\n  Decimal (.) on NUMERIC keypad: Circulate Fill Shape of Cell from, histogram/Full/None.";
			    msg=msg+"\r\n  Multiply/Divide ('*' '/') on NUMERIC keypad: Circulate Forward/Backward Center Ladder Type";
			    msg=msg+"\r\n  Add/Subtract (+/-) on NUMERIC keypad: Increase/Decrease Aggreg. Factor";
			    msg=msg+"\r\n  Ctrl + F4: Circulate Cell Color Scheme from MaximumOfLadder/FixedNumbers/DominantSide";
			    msg=msg+"\r\n  Shift + F2: Center BA High Side Mode Toggle";
				msg=msg+"\r\n  Shift + F5: Center BA Borders on/off";
				msg=msg+"\r\n  Shift + F6: Center BA Text on/off";
				msg=msg+"\r\n  Shift + F7: Side Ladders Text on/off";
				msg=msg+"\r\n  Shift + F8: Side Ladders Borders on/off";
				msg=msg+"\r\n  Shift +F9: Circulate Bottom View : None/Statistic/Graphic";
			    msg=msg+"\r\n  Shift +F12: BA True Shape  Toggle";
				msg=msg+"\r\n  Alt + F10: Show/Hide COT Numbers";
				msg=msg+"\r\n  Alt + F11: Show/Hide Contract Reversal";
				msg=msg+"\r\n  Alt + F12: Show Abbreviations in Lower Boxes";
				msg=msg+"\r\n  *********************************";
			    msg=msg+"\r\n  Center Ladder Type: " + helper.clt + "  " + (int)helper.clt;
			    msg=msg+"\r\n  CustomCenterLadder Component: " + "  Base:" + helper.clb + "  Shape:"+helper.cls+"  Data:"+helper.cld;
			    msg=msg+"\r\n  Cell Color Scheme: " + helper.ccs + "  " + (int)helper.ccs;
				msg=msg+"\r\n  *********************************";
			
			if(helper.showhotkeys){
			
			graphics.FillRectangle(new SolidBrush(ChartControl.BackColor),bounds.Left,bounds.Bottom-330,graphics.MeasureString(msg,ChartControl.Font).Width,graphics.MeasureString(msg,ChartControl.Font).Height);
			graphics.DrawString(msg, ChartControl.Font, new SolidBrush(ChartControl.AxisColor), bounds.Left, bounds.Bottom-330,new StringFormat());
				}
			
            } catch (Exception ex){
			Print("GomLadder PlotGraphics: " + ex.Message);
				return;
			}	
		}
		#endregion
		
		#region GomOnTermination
		protected override void GomOnTermination()
		{
			if (keyEvtH != null)
			{	
				this.ChartControl.ChartPanel.KeyDown -= keyEvtH;
			   	keyEvtH=null;
			}
			
			this.ChartControl.ChartStyle.UpColor = cup;
			this.ChartControl.ChartStyle.DownColor = cdn;
			this.CandleOutlineColor = ccandle;
			
		}
		#endregion

        #region Properties
		[Browsable(false)]
		//[XmlIgnore()]
		public int InitBarWidth
			
			{
			get { return initBarWidth; }
			set {  initBarWidth=value;}
		}
			
		[Browsable(false)]
		//[XmlIgnore()]
		public int InitBarSpace
			
			{
			get { return initBarSpace; }
			set {  initBarSpace=value;}
		}
			
		  #region COLORS
		[XmlIgnore()]
		[Description("Font Color of BA Ladders")]
		[Category("06. Colors")]
		[Gui.Design.DisplayName("BA Font Color")]
		
		public Color BaFontColor
		{
			get { return (helper.baFontColor); }
			set {  helper.baFontColor=value;}
		}
		
		[Browsable(false)]
		public string BaFontColorSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.baFontColor); }
     		set { helper.baFontColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		
		[XmlIgnore()]
		[Description("Font Color of Side Ladders")]
		[Category("06. Colors")]
		[Gui.Design.DisplayName("Side Font Color")]
		
		public Color SideFontColor
		{
			get { return (helper.sideFontColor); }
			set {  helper.sideFontColor=value;}
		}
		
		[Browsable(false)]
		public string SideFontColorSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.sideFontColor); }
     		set { helper.sideFontColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		
		[XmlIgnore()]
		[Description("Font Color of Bottom Boxes ")]
		[Category("06. Colors")]
		[Gui.Design.DisplayName("Bottom Font Color")]
		
		public Color BottomFontColor
		{
			get { return (helper.bottomFontColor); }
			set {  helper.bottomFontColor=value;}
		}
		
		[Browsable(false)]
		public string BottomFontColorSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.bottomFontColor); }
     		set { helper.bottomFontColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		
		[XmlIgnore()]
		[Description("Color of Legend BackGround Fill ")]
		[Category("06. Colors")]
		[Gui.Design.DisplayName("Legend Back Color")]
		
		public Color LegendBackColor
		{
			get { return (helper.legendBackColor); }
			set {  helper.legendBackColor=value;}
		}
		
		[Browsable(false)]
		public string LegendBackColorSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.legendBackColor); }
     		set { helper.legendBackColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
			
		
		[XmlIgnore()]
		[Description("Color on ask and positive values")]
		[Category("06. Colors")]
		[Gui.Design.DisplayName("Positive")]
		
		public Color ColorUp
		{
			get { return (helper.colorUp); }
			set {  helper.colorUp=value;}
		}
		
		[Browsable(false)]
		public string ColorUpSerialize
			{
    			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.colorUp); }
     			set { helper.colorUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
			}
			
		
			
		[XmlIgnore()]
		[Description("Color on bid and negative values")]
		[Category("06. Colors")]
		[Gui.Design.DisplayName("Negative")]
		
		public Color ColorDown
		{
			get { return helper.colorDown; }
			set {  helper.colorDown=value;}
		}
		
		[Browsable(false)]
		public string ColorDownSerialize
			{
    			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.colorDown); }
     			set { helper.colorDown = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
			}	
			
			
		
		[Description("Color of the volume bars in side ladder")]
		[Category("06. Colors")]
		[Gui.Design.DisplayName("Volume Bar, Side Ladders")]
		
		[XmlIgnore()]
		public Color ColorSideLadderVolume
		{
			get { return helper.colorSideLadderVolume; }
			set {  helper.colorSideLadderVolume=value;}
		}
		
		[Browsable(false)]
		public string ColorSideLadderVolumeSerialize
			{
    			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.colorSideLadderVolume); }
     			set { helper.colorSideLadderVolume = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
			}
		
		[Description("Color of the volume bars in BA")]
		[Category("06. Colors")]
		[Gui.Design.DisplayName("Volume Bar, Center Ladder")]
		
		[XmlIgnore()]
		public Color ColorCenterLadderVolume
		{
			get { return helper.colorCenterLadderVolume; }
			set {  helper.colorCenterLadderVolume=value;}
		}
		
		[Browsable(false)]
		public string ColorCenterLadderVolumeSerialize
			{
    			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.colorCenterLadderVolume); }
     			set { helper.colorCenterLadderVolume = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
			}
			
			
		[Description("Color of ladder background")]
		[Category("06. Colors")]
		[Gui.Design.DisplayName("Background")]
		
		[XmlIgnore()]
		public Color ColorBackground
		{
			get { return helper.colorBackGround; }
			set {  
				helper.colorBackGround=value;
				helper.myWhiteBrush.Color=helper.colorBackGround;
				}
		}
		
		[Browsable(false)]
		public string ColorBackGroundSerialize
			{
    			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.colorBackGround); }
     			set { helper.myWhiteBrush.Color = helper.colorBackGround = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }	// bug fix 4-Jul-2010
			}
		#endregion
			
		  #region DiagonalBidAsk/Custom Candle
		[XmlIgnore()]
		[Description("Diagonal Strong Ask Color")]
		[Category("12. Diagonal Bid-Ask & Custom Candle")]
		[Gui.Design.DisplayName("01. Strong Ask Color")]
		
		public Color DiaStrongAskColor
		{
			get { return (helper.DiaStrongAskColor); }
			set {  helper.DiaStrongAskColor=value;}
		}
		
		[Browsable(false)]
		public string DiaStrongAskColorSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.DiaStrongAskColor); }
     		set { helper.DiaStrongAskColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		
		
		[XmlIgnore()]
		[Description("Diagonal Strong Bid Color")]
		[Category("12. Diagonal Bid-Ask & Custom Candle")]
		[Gui.Design.DisplayName("02. Strong Bid Color")]
		
		public Color DiaStrongBidColor
		{
			get { return (helper.DiaStrongBidColor); }
			set {  helper.DiaStrongBidColor=value;}
		}
		
		[Browsable(false)]
		public string DiaStrongBidColorSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.DiaStrongBidColor); }
     		set { helper.DiaStrongBidColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		
		[XmlIgnore()]
		[Description("Diagonal Weak Color")]
		[Category("12. Diagonal Bid-Ask & Custom Candle")]
		[Gui.Design.DisplayName("03. Weak Color")]
		
		public Color DiaWeakColor
		{
			get { return (helper.DiaWeakColor); }
			set {  helper.DiaWeakColor=value;}
		}
		
		[Browsable(false)]
		public string DiaWeakColorSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.DiaWeakColor); }
     		set { helper.DiaWeakColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		
		[XmlIgnore()]
		[Description("Diagonal Neutral Color")]
		[Category("12. Diagonal Bid-Ask & Custom Candle")]
		[Gui.Design.DisplayName("04. Neutral Color")]
		
		public Color DiaNeutralColor
		{
			get { return (helper.DiaNeutralColor); }
			set {  helper.DiaNeutralColor=value;}
		}
		
		[Browsable(false)]
		public string DiaNeutralColorSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.DiaNeutralColor); }
     		set { helper.DiaNeutralColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		
		[XmlIgnore()]
		[Description("Select Diagonal Bid-Ask Font")]
		[Category("12. Diagonal Bid-Ask & Custom Candle")]
		[Gui.Design.DisplayName("05. Dia Font")]
		public Font DiaFont		
		{
			get { return helper.diaFont; }
			set { helper.diaFont = value; }
		}	
		
		[Browsable(false)]
		public string DiaFontSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableFont.ToString(helper.diaFont); }
     		set { helper.diaFont = NinjaTrader.Gui.Design.SerializableFont.FromString(value); }
		}
		
		
		[Description("Threshold Volume(or Tick in case of CellDataType is Tick) above which Bid/Ask will be considered for strength comparision")]
		[Category("12. Diagonal Bid-Ask & Custom Candle")]
		[Gui.Design.DisplayName("06. Dia Threshold Volume")]
		public int DiaThresholdVolume
			
		{
			get { return helper.diagonalThresholdVolume; }
			set {  helper.diagonalThresholdVolume=Math.Max(1,value);}
		}
		
		
		[Description("Numerator (int) in strength comparision fraction")]
		[Category("12. Diagonal Bid-Ask & Custom Candle")]
		[Gui.Design.DisplayName("07. Numerator")]
		public int DiaNumerator
			
		{
			get { return helper.diaNumerator; }
			set {  helper.diaNumerator=Math.Max(1,value);}
		}
		
		
		[Description("Denominator (int) in strength comparision fraction")]
		[Category("12. Diagonal Bid-Ask & Custom Candle")]
		[Gui.Design.DisplayName("08. Denominator")]
		public int DiaDenominator
			
		{
			get { return helper.diaDenominator; }
			set {  helper.diaDenominator=Math.Max(1,value);}
		}
		
		
		[XmlIgnore()]
		[Description("Custom Candle Up Color")]
		[Category("12. Diagonal Bid-Ask & Custom Candle")]
		[Gui.Design.DisplayName("09. Candle Up Color")]
		
		public Color CandleUpColor
		{
			get { return (helper.customCandleUpColor); }
			set {  helper.customCandleUpColor=value;}
		}
		
		[Browsable(false)]
		public string CandleUpColorSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.customCandleUpColor); }
     		set { helper.customCandleUpColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		
		[XmlIgnore()]
		[Description("Custom Candle Down Color")]
		[Category("12. Diagonal Bid-Ask & Custom Candle")]
		[Gui.Design.DisplayName("10. Candle Dn Color")]
		
		public Color CandleDnColor
		{
			get { return (helper.customCandleDnColor); }
			set {  helper.customCandleDnColor=value;}
		}
		
		[Browsable(false)]
		public string CandleDnColorSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.customCandleDnColor); }
     		set { helper.customCandleDnColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		
		[XmlIgnore()]
		[Description("Custom Candle Neutral Color")]
		[Category("12. Diagonal Bid-Ask & Custom Candle")]
		[Gui.Design.DisplayName("11. Candle Neutral Color")]
		
		public Color CandleNeutralColor
		{
			get { return (helper.customCandleNeutralColor); }
			set {  helper.customCandleNeutralColor=value;}
		}
		
		[Browsable(false)]
		public string CandleNeutralColorSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.customCandleNeutralColor); }
     		set { helper.customCandleNeutralColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		
		[Description("Custom Candle Fill Opacity ( 0 - 255 )")]
		[Category("12. Diagonal Bid-Ask & Custom Candle")]
		[Gui.Design.DisplayName("12. Candle Fill Opacity")]
		public int CandleFillOpacity
			
		{
			get { return helper.customCandleFillOpacity; }
			set {  helper.customCandleFillOpacity=Math.Max(0,Math.Min(255,value));}
		}
		#endregion
		
		
		  #region CustomCenterLadder Setting
		[Description("CustomCenterLadder Base (Histogram Color will be according to it), select from Volume or Delta")]
		[Category("13. Custom Canter Ladder")]
		[Gui.Design.DisplayName("01. Base")]

		public CustomLadderBase Clb
		{
			get {return helper.clb;}
			set {helper.clb=value;}
		}
		
		
		[Description("CustomCenterLadder Shape (Histogram length will be accroding to it), select from Volume or Delta")]
		[Category("13. Custom Canter Ladder")]
		[Gui.Design.DisplayName("02. Shape")]

		public CustomLadderShape Cls
		{
			get {return helper.cls;}
			set {helper.cls=value;}
		}
		
		
		[Description("CustomCenterLadder Data (Data to show on Ladder), select from Volume or Delta or BidxAsk")]
		[Category("13. Custom Canter Ladder")]
		[Gui.Design.DisplayName("03. Data")]

		public CustomLadderData Cld
		{
			get {return helper.cld;}
			set {helper.cld=value;}
		}
		
		
		[Description("CustomCenterLadder Histogram Direction")]
		[Category("13. Custom Canter Ladder")]
		[Gui.Design.DisplayName("04. Histogram Direction")]

		public Direction Cldirection
		{
			get {return helper.clDirection;}
			set {helper.clDirection=value;}
		}
		
		
		[Description("CustomCenterLadder Candle Location. LeftToRight -> Left Candle, RightToLeft -> Right Candle")]
		[Category("13. Custom Canter Ladder")]
		[Gui.Design.DisplayName("04. Candle Location")]

		public Direction CandleDirection
		{
			get {return helper.candleDirection;}
			set {helper.candleDirection=value;}
		}
		  #endregion
			
		  #region PENS
		[Editor((typeof(PenEditor)), (typeof(UITypeEditor)))]
		[TypeConverter((typeof(PenConverter)))]
		[XmlIgnore()]
		[Description("Set the color/thickness/style for Opening Tick of Ladder")]
		[Category("11. Pen")]
		[Gui.Design.DisplayNameAttribute("01. Open Pen Style")]
		public Pen MyOpenPen
		{
			get { return helper.myOpenPen; }
			set { helper.myOpenPen = value; }
		}

		[Browsable(false)]
		public SerializablePen MyOpenPenSerialize
		{
			get { return SerializablePen.FromPen(helper.myOpenPen); }
			set { helper.myOpenPen = SerializablePen.ToPen(value); }
		}
		
		
		[Editor((typeof(PenEditor)), (typeof(UITypeEditor)))]
		[TypeConverter((typeof(PenConverter)))]
		[XmlIgnore()]
		[Description("Set the color/thickness/style for Last Tick of Ladder")]
		[Category("11. Pen")]
		[Gui.Design.DisplayNameAttribute("02. Tick Pen Style")]
		public Pen MyTickPen
		{
			get { return helper.myTickPen; }
			set { helper.myTickPen = value; }
		}

		[Browsable(false)]
		public SerializablePen MyTickPenSerialize
		{
			get { return SerializablePen.FromPen(helper.myTickPen); }
			set { helper.myTickPen = SerializablePen.ToPen(value); }
		}
		
		
		[Editor((typeof(PenEditor)), (typeof(UITypeEditor)))]
		[TypeConverter((typeof(PenConverter)))]
		[XmlIgnore()]
		[Description("Set the color/thickness/style for Highest Volume Tick Price of Ladder")]
		[Category("11. Pen")]
		[Gui.Design.DisplayNameAttribute("03. High Vol Style")]
		public Pen MyVolPen
		{
			get { return helper.myVolPen; }
			set { helper.myVolPen = value; }
		}

		[Browsable(false)]
		public SerializablePen MyVolPenSerialize
		{
			get { return SerializablePen.FromPen(helper.myVolPen); }
			set { helper.myVolPen = SerializablePen.ToPen(value); }
		}
		
		[Editor((typeof(PenEditor)), (typeof(UITypeEditor)))]
		[TypeConverter((typeof(PenConverter)))]
		[XmlIgnore()]
		[Description("Set the color/thickness/style for 2nd Highest Volume Tick Price of Ladder")]
		[Category("11. Pen")]
		[Gui.Design.DisplayNameAttribute("04. 2nd High Vol Style")]
		public Pen MyVol2Pen
		{
			get { return helper.myVol2Pen; }
			set { helper.myVol2Pen = value; }
		}

		[Browsable(false)]
		public SerializablePen MyVol2PenSerialize
		{
			get { return SerializablePen.FromPen(helper.myVol2Pen); }
			set { helper.myVol2Pen = SerializablePen.ToPen(value); }
		}
		
		
		[Editor((typeof(PenEditor)), (typeof(UITypeEditor)))]
		[TypeConverter((typeof(PenConverter)))]
		[XmlIgnore()]
		[Description("Set the color/thickness/style for BA Cell Borders of Ladder")]
		[Category("11. Pen")]
		[Gui.Design.DisplayNameAttribute("05. BA Border Style")]
		public Pen MyBAborderPen
		{
			get { return helper.myBAborderPen; }
			set { helper.myBAborderPen = value; }
		}

		[Browsable(false)]
		public SerializablePen MyBAborderPenSerialize
		{
			get { return SerializablePen.FromPen(helper.myBAborderPen); }
			set { helper.myBAborderPen = SerializablePen.ToPen(value); }
		}
		
		
		[Editor((typeof(PenEditor)), (typeof(UITypeEditor)))]
		[TypeConverter((typeof(PenConverter)))]
		[XmlIgnore()]
		[Description("Set the color/thickness/style for VWAP")]
		[Category("11. Pen")]
		[Gui.Design.DisplayNameAttribute("06. VWAP Style")]
		public Pen MyVwapPen
		{
			get { return helper.myVwapPen; }
			set { helper.myVwapPen = value; }
		}

		[Browsable(false)]
		public SerializablePen MyVwapPenSerialize
		{
			get { return SerializablePen.FromPen(helper.myVwapPen); }
			set { helper.myVwapPen = SerializablePen.ToPen(value); }
		}
		
		  #endregion
			
		  #region E Left Ladder
		
		[Description("")]
		[Category("01. Extreme Left Ladder")]
		[Gui.Design.DisplayName("01. Show")]
		
		public bool ELshow
		{
			get { return gLadderExtremeLeft.Show; }
			set {   gLadderExtremeLeft.Show=value;}
		}		
		
		[Description("")]
		[Category("01. Extreme Left Ladder")]
		[Gui.Design.DisplayName("02. Ladder Type")]
		
		public LadderTypeEnum ELLadderType
		{
			get { return gLadderExtremeLeft.LadderType; }
			set {   gLadderExtremeLeft.LadderType=value;}
		}	
		
		[Description("Determines how the ladder is initialzed in the begginning of each session. It's the sum of 0 to max number of previous session lose ladders")]
		[Category("01. Extreme Left Ladder")]
		[Gui.Design.DisplayName("03. Accumulation Type")]
		
		public AccumulationTypeEnum ELAccumulationType
		{
			get { return gLadderExtremeLeft.AccumulationType; }
			set {  gLadderExtremeLeft.AccumulationType=value;}
		}	

		[Description("Used only if above type is nDays")]
		[Category("01. Extreme Left Ladder")]
		[Gui.Design.DisplayName("04. Accumulation Days")]
		
		public int ELAccumulationDays
		{
			get { return gLadderExtremeLeft.AccumulationDays; }
			set {  gLadderExtremeLeft.AccumulationDays=value;}
		}
		
		[Description("")]
		[Category("01. Extreme Left Ladder")]
		[Gui.Design.DisplayName("05. Number Format")]
		
		public FormattingType ELNumberFormat
		{
			get { return gLadderExtremeLeft.LadderFormattingType; }
			set {  gLadderExtremeLeft.LadderFormattingType=value;}
		}
		
		[Description("Determines how are alpha blended colors and sized rectangles. Max value of current ladder or fixes value")]
		[Category("01. Extreme Left Ladder")]
		[Gui.Design.DisplayName("06. Ratio Type")]

		public RatioType ELRatioType
		{
			get { return gLadderExtremeLeft.LadderRatioType; }
			set {  gLadderExtremeLeft.LadderRatioType=value;}
		}
		
		[Description("Used only if above is fixed number. Determines max value expected. Above values are displayed 100%")]
		[Category("01. Extreme Left Ladder")]
		[Gui.Design.DisplayName("07. Ratio Max Number")]

		public int ELRatioNumber
		{
			get { return gLadderExtremeLeft.LadderFixedNumber; }
			set {  gLadderExtremeLeft.LadderFixedNumber=value;}
		}
		
		
		[Description("Side Ladder Histogram Type (None/Histogram/Full)")]
		[Category("01. Extreme Left Ladder")]
		[Gui.Design.DisplayName("08. Cell Fill Type")]

		public CellFillType ELCellFillType
		{
			get { return gLadderExtremeLeft.GCellFillType; }
			set {  gLadderExtremeLeft.GCellFillType=value;}
		}
		
		
		[Description("Side Ladder Histogram Type Direction")]
		[Category("01. Extreme Left Ladder")]
		[Gui.Design.DisplayName("09. Histogram Direction")]

		public Direction ELDirection
		{
			get { return gLadderExtremeLeft.GDirection; }
			set {  gLadderExtremeLeft.GDirection=value;}
		}
		
		[Description("Side Ladder Data Location in Cell")]
		[Category("01. Extreme Left Ladder")]
		[Gui.Design.DisplayName("10. Cell Data Location")]

		public CellDataLocation ELDataLocation
		{
			get { return gLadderExtremeLeft.GDataLocation; }
			set {  gLadderExtremeLeft.GDataLocation=value;}
		}
		
		[Description("Side Ladder Width")]
		[Category("01. Extreme Left Ladder")]
		[Gui.Design.DisplayName("11. Ladder Width")]

		public SideLadderWidth ELCellWidth
		{
			get { return gLadderExtremeLeft.GWidth; }
			set {  gLadderExtremeLeft.GWidth=value;}
		}
		
		
		[Description("Show Data ( numbers ) on Ladder Cells")]
		[Category("01. Extreme Left Ladder")]
		[Gui.Design.DisplayName("12. Show Cell Data")]
		
		public bool ELShowCellData
		{
			get { return gLadderExtremeLeft.GShowCellData; }
			set {   gLadderExtremeLeft.GShowCellData=value;}
		}	
		
		#endregion
		
		  #region Left Ladder
		
		[Description("")]
		[Category("02. Left Ladder")]
		[Gui.Design.DisplayName("01. Show")]
		
		public bool Lshow
		{
			get { return gLadderLeft.Show; }
			set {   gLadderLeft.Show=value;}
		}		
		
		[Description("")]
		[Category("02. Left Ladder")]
		[Gui.Design.DisplayName("02. Ladder Type")]
		
		public LadderTypeEnum LLadderType
		{
			get { return gLadderLeft.LadderType; }
			set {   gLadderLeft.LadderType=value;}
		}	
		
		[Description("Determines how the ladder is initialzed in the begginning of each session. It's the sum of 0 to max number of previous session lose ladders")]
		[Category("02. Left Ladder")]
		[Gui.Design.DisplayName("03. Accumulation Type")]
		
		public AccumulationTypeEnum LAccumulationType
		{
			get { return gLadderLeft.AccumulationType; }
			set {  gLadderLeft.AccumulationType=value;}
		}	

		[Description("Used only if above type is nDays")]
		[Category("02. Left Ladder")]
		[Gui.Design.DisplayName("04. Accumulation Days")]
		
		public int LAccumulationDays
		{
			get { return gLadderLeft.AccumulationDays; }
			set {  gLadderLeft.AccumulationDays=value;}
		}
		
		[Description("")]
		[Category("02. Left Ladder")]
		[Gui.Design.DisplayName("05. Number Format")]
		
		public FormattingType LNumberFormat
		{
			get { return gLadderLeft.LadderFormattingType; }
			set {  gLadderLeft.LadderFormattingType=value;}
		}
		
		[Description("Determines how are alpha blended colors and sized rectangles. Max value of current ladder or fixes value")]
		[Category("02. Left Ladder")]
		[Gui.Design.DisplayName("06. Ratio Type")]

		public RatioType LRatioType
		{
			get { return gLadderLeft.LadderRatioType; }
			set {  gLadderLeft.LadderRatioType=value;}
		}
		
		[Description("Used only if above is fixed number. Determines max value expected. Above values are displayed 100%")]
		[Category("02. Left Ladder")]
		[Gui.Design.DisplayName("07. Ratio Max Number")]

		public int LRatioNumber
		{
			get { return gLadderLeft.LadderFixedNumber; }
			set {  gLadderLeft.LadderFixedNumber=value;}
		}
		
		
		[Description("Side Ladder Histogram Type (None/Histogram/Full)")]
		[Category("02. Left Ladder")]
		[Gui.Design.DisplayName("08. Cell Fill Type")]

		public CellFillType LCellFillType
		{
			get { return gLadderLeft.GCellFillType; }
			set {  gLadderLeft.GCellFillType=value;}
		}
		
		
		[Description("Side Ladder Histogram Type Direction")]
		[Category("02. Left Ladder")]
		[Gui.Design.DisplayName("09. Histogram Direction")]

		public Direction LDirection
		{
			get { return gLadderLeft.GDirection; }
			set {  gLadderLeft.GDirection=value;}
		}
		
		[Description("Side Ladder Data Location in Cell")]
		[Category("02. Left Ladder")]
		[Gui.Design.DisplayName("10. Cell Data Location")]

		public CellDataLocation LDataLocation
		{
			get { return gLadderLeft.GDataLocation; }
			set {  gLadderLeft.GDataLocation=value;}
		}
		
		[Description("Side Ladder Width")]
		[Category("02. Left Ladder")]
		[Gui.Design.DisplayName("11. Ladder Width")]

		public SideLadderWidth LCellWidth
		{
			get { return gLadderLeft.GWidth; }
			set {  gLadderLeft.GWidth=value;}
		}
		
		[Description("Show Data ( numbers ) on Ladder Cells")]
		[Category("02. Left Ladder")]
		[Gui.Design.DisplayName("12. Show Cell Data")]
		
		public bool LShowCellData
		{
			get { return gLadderLeft.GShowCellData; }
			set {   gLadderLeft.GShowCellData=value;}
		}	
		#endregion
		
		  #region Right Ladder
		[Description("")]
		[Category("03. Right Ladder")]
		[Gui.Design.DisplayName("01. Show")]
		
		public bool Rshow
		{
			get { return gLadderRight.Show; }
			set {   gLadderRight.Show=value;}
		}		
		
		[Description("")]
		[Category("03. Right Ladder")]
		[Gui.Design.DisplayName("02. Ladder Type")]
		
		public LadderTypeEnum RLadderType
		{
			get { return gLadderRight.LadderType; }
			set {   gLadderRight.LadderType=value;}
		}	
		
		[Description("Determines how the ladder is initialzed in the begginning of each session. It's the sum of 0 to max number of previous session lose ladders")]
		[Category("03. Right Ladder")]
		[Gui.Design.DisplayName("03. Accumulation Type")]
		
		public AccumulationTypeEnum RAccumulationType
		{
			get { return gLadderRight.AccumulationType; }
			set {  gLadderRight.AccumulationType=value;}
		}	

		[Description("Used only if above type is nDays")]
		[Category("03. Right Ladder")]
		[Gui.Design.DisplayName("04. Accumulation Days")]
		
		public int RAccumulationDays
		{
			get { return gLadderRight.AccumulationDays; }
			set {  gLadderRight.AccumulationDays=value;}
		}
		
		[Description("")]
		[Category("03. Right Ladder")]
		[Gui.Design.DisplayName("05. Number Format")]
		
		public FormattingType RNumberFormat
		{
			get { return gLadderRight.LadderFormattingType; }
			set {  gLadderRight.LadderFormattingType=value;}
		}
		
		[Description("Determines how are alpha blended colors and sized rectangles. Max value of current ladder or fixes value")]
		[Category("03. Right Ladder")]
		[Gui.Design.DisplayName("06. Ratio Type")]

		public RatioType RRatioType
		{
			get { return gLadderRight.LadderRatioType; }
			set {  gLadderRight.LadderRatioType=value;}
		}
		
		[Description("Used only if above is fixed number. Determines max value expected. Above values are displayed 100%")]
		[Category("03. Right Ladder")]
		[Gui.Design.DisplayName("07. Ratio Max Number")]

		public int RRatioNumber
		{
			get { return gLadderRight.LadderFixedNumber; }
			set {  gLadderRight.LadderFixedNumber=value;}
		}
		
		
		[Description("Side Ladder Histogram Type (None/Histogram/Full)")]
		[Category("03. Right Ladder")]
		[Gui.Design.DisplayName("08. Cell Fill Type")]

		public CellFillType RCellFillType
		{
			get { return gLadderRight.GCellFillType; }
			set {  gLadderRight.GCellFillType=value;}
		}
		
		
		[Description("Side Ladder Histogram Type Direction")]
		[Category("03. Right Ladder")]
		[Gui.Design.DisplayName("09. Histogram Direction")]

		public Direction RDirection
		{
			get { return gLadderRight.GDirection; }
			set {  gLadderRight.GDirection=value;}
		}
		
		[Description("Side Ladder Data Location in Cell")]
		[Category("03. Right Ladder")]
		[Gui.Design.DisplayName("10. Cell Data Location")]

		public CellDataLocation RDataLocation
		{
			get { return gLadderRight.GDataLocation; }
			set {  gLadderRight.GDataLocation=value;}
		}
		
		[Description("Side Ladder Width")]
		[Category("03. Right Ladder")]
		[Gui.Design.DisplayName("11. Ladder Width")]

		public SideLadderWidth RCellWidth
		{
			get { return gLadderRight.GWidth; }
			set {  gLadderRight.GWidth=value;}
		}
		
		
		[Description("Show Data ( numbers ) on Ladder Cells")]
		[Category("03. Right Ladder")]
		[Gui.Design.DisplayName("12. Show Cell Data")]
		
		public bool RShowCellData
		{
			get { return gLadderRight.GShowCellData; }
			set {   gLadderRight.GShowCellData=value;}
		}	
		#endregion
		
		  #region E Right Ladder
		
		[Description("")]
		[Category("04. Extreme Right Ladder")]
		[Gui.Design.DisplayName("01. Show")]
		
		public bool ERshow
		{
			get { return gLadderExtremeRight.Show; }
			set {   gLadderExtremeRight.Show=value;}
		}		
		
		[Description("")]
		[Category("04. Extreme Right Ladder")]
		[Gui.Design.DisplayName("02. Ladder Type")]
		
		public LadderTypeEnum ERLadderType
		{
			get { return gLadderExtremeRight.LadderType; }
			set {   gLadderExtremeRight.LadderType=value;}
		}	
		
		[Description("Determines how the ladder is initialzed in the begginning of each session. It's the sum of 0 to max number of previous session lose ladders")]
		[Category("04. Extreme Right Ladder")]
		[Gui.Design.DisplayName("03. Accumulation Type")]
		
		public AccumulationTypeEnum ERAccumulationType
		{
			get { return gLadderExtremeRight.AccumulationType; }
			set {  gLadderExtremeRight.AccumulationType=value;}
		}	

		[Description("Used only if above type is nDays")]
		[Category("04. Extreme Right Ladder")]
		[Gui.Design.DisplayName("04. Accumulation Days")]
		
		public int ERAccumulationDays
		{
			get { return gLadderExtremeRight.AccumulationDays; }
			set {  gLadderExtremeRight.AccumulationDays=value;}
		}
		
		[Description("")]
		[Category("04. Extreme Right Ladder")]
		[Gui.Design.DisplayName("05. Number Format")]
		
		public FormattingType ERNumberFormat
		{
			get { return gLadderExtremeRight.LadderFormattingType; }
			set {  gLadderExtremeRight.LadderFormattingType=value;}
		}
		
		[Description("Determines how are alpha blended colors and sized rectangles. Max value of current ladder or fixes value")]
		[Category("04. Extreme Right Ladder")]
		[Gui.Design.DisplayName("06. Ratio Type")]

		public RatioType ERRatioType
		{
			get { return gLadderExtremeRight.LadderRatioType; }
			set {  gLadderExtremeRight.LadderRatioType=value;}
		}
		
		[Description("Used only if above is fixed number. Determines max value expected. Above values are displayed 100%")]
		[Category("04. Extreme Right Ladder")]
		[Gui.Design.DisplayName("07. Ratio Max Number")]

		public int ERRatioNumber
		{
			get { return gLadderExtremeRight.LadderFixedNumber; }
			set {  gLadderExtremeRight.LadderFixedNumber=value;}
		}
		
		
	    [Description("Side Ladder Histogram Type (None/Histogram/Full)")]
		[Category("04. Extreme Right Ladder")]
		[Gui.Design.DisplayName("08. Cell Fill Type")]

		public CellFillType ERCellFillType
		{
			get { return gLadderExtremeRight.GCellFillType; }
			set {  gLadderExtremeRight.GCellFillType=value;}
		}
		
		
		[Description("Side Ladder Histogram Type Direction")]
		[Category("04. Extreme Right Ladder")]
		[Gui.Design.DisplayName("09. Histogram Direction")]

		public Direction ERDirection
		{
			get { return gLadderExtremeRight.GDirection; }
			set {  gLadderExtremeRight.GDirection=value;}
		}
		
		[Description("Side Ladder Data Location in Cell")]
		[Category("04. Extreme Right Ladder")]
		[Gui.Design.DisplayName("10. Cell Data Location")]

		public CellDataLocation ERDataLocation
		{
			get { return gLadderExtremeRight.GDataLocation; }
			set {  gLadderExtremeRight.GDataLocation=value;}
		}
		
		[Description("Side Ladder Width")]
		[Category("04. Extreme Right Ladder")]
		[Gui.Design.DisplayName("11. Ladder Width")]

		public SideLadderWidth ERCellWidth
		{
			get { return gLadderExtremeRight.GWidth; }
			set {  gLadderExtremeRight.GWidth=value;}
		}
		
		[Description("Show Data ( numbers ) on Ladder Cells")]
		[Category("04. Extreme Right Ladder")]
		[Gui.Design.DisplayName("12. Show Cell Data")]
		
		public bool ERShowCellData
		{
			get { return gLadderExtremeRight.GShowCellData; }
			set {   gLadderExtremeRight.GShowCellData=value;}
		}	
		#endregion
		
		  #region Settings
		[Description("Lot Size, to be used as number of share per lot in LotSize formatting")]
		[Category("Settings")]
		[Gui.Design.DisplayName("01. LotSize")]
		public int LotSize
			
			{
			get { return helper.lotSize; }
			set {  helper.lotSize=Math.Max(1,value);}
		}
		
		
		[Description("Determines initial Fib aggregation Level of the cells")]
		[Category("Settings")]
		[Gui.Design.DisplayName("02. Aggregation Fib Level")]

		public int FibLevel
		{
			get {return helper.agregationFib;}
			set {helper.agregationFib=value;
				 helper.agregationFactor=Fibonacci(helper.agregationFib);}
			
		}
		
		
		[Description("Histogram Spacing BA Ladder")]
		[Category("Settings")]
		[Gui.Design.DisplayName("03. Histogram Spacing BA Ladder Set from .10-1, 1 Equals No Spacing")]

		public double HistogramSpacing
		{
			get { return helper.BALadderHistogramSpacing; }
			set {  helper.BALadderHistogramSpacing=value;}
		}
		
		
		
		[Description("Histogram Spacing Side Ladder")]
		[Category("Settings")]
		[Gui.Design.DisplayName("04. Histogram Spacing Side Ladder Set from .10-1, 1 Equals No Spacing")]

		public double SideHistogramSpacing
		{
			get { return helper.SideLadderHistogramSpacing; }
			set {  helper.SideLadderHistogramSpacing=value;}
		}
		
		
		
		[Description("To spare memory, in live mode there is 1 ladder per session, not per bar (except for BA ladders of course)")]
		[Category("Settings")]
		[Gui.Design.DisplayName("05. Live Mode")]

		public bool LiveMode
		{
			get {return helper.liveMode;}
			set {helper.liveMode=value;}
		}
		#endregion
		
	      #region BottomGraphicSettingsAndColors
		[Description("Select Bottom Graphic Type ")]
		[Category("10. Bottom Graphic")]
		[Gui.Design.DisplayName("01. Bottom Graphic Type")]

		public BottomGraphicType  BGType
		{
			get { return helper.bottomgraphic; }
			set {  helper.bottomgraphic=value;}
		}
		
		
		[Description("Select Bottom Graphic Height ")]
		[Category("10. Bottom Graphic")]
		[Gui.Design.DisplayName("02. Bottom Graphic Height")]

		public BottomGraphicHeight  BGHType
		{
			get { return helper.bgh; }
			set {  helper.bgh=value;}
		}
		
		
		[Description("Select Graphic Base Alpha, 1 - 255")]
		[Category("10. Bottom Graphic")]
		[Gui.Design.DisplayName("03. Base Opacity")]
		public int BGBaseOpacity
			
			{
			get { return helper.bgBaseOpacity; }
			set {  helper.bgBaseOpacity=Math.Max(1,Math.Min(255,value));}
		}	
		
		
		[XmlIgnore()]
		[Description("Base Color for Bottom Graphic")]
		[Category("10. Bottom Graphic")]
		[Gui.Design.DisplayName("04.  Base Color")]
		
		public Color BGBaseColor
		{
			get { return (helper.colorBottomGraphicBase); }
			set {  helper.colorBottomGraphicBase=value;}
		}
		
		[Browsable(false)]
		public string BGBaseColorSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.colorBottomGraphicBase); }
     		set { helper.colorBottomGraphicBase = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[XmlIgnore()]
		[Description("Volume Bar Color for Bottom Graphic")]
		[Category("10. Bottom Graphic")]
		[Gui.Design.DisplayName("05.  Volume Bar Color")]
		
		public Color BGVolColor
		{
			get { return (helper.colorBottomGraphicVol); }
			set {  helper.colorBottomGraphicVol=value;}
		}
		
		[Browsable(false)]
		public string BGVolColorSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.colorBottomGraphicVol); }
     		set { helper.colorBottomGraphicVol = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		
		[XmlIgnore()]
		[Description("Bid Volume Color for Bottom Graphic")]
		[Category("10. Bottom Graphic")]
		[Gui.Design.DisplayName("06.  Bid Volume Color")]
		
		public Color BGBidColor
		{
			get { return (helper.colorBottomGraphicBid); }
			set {  helper.colorBottomGraphicBid=value;}
		}
		
		[Browsable(false)]
		public string BGBidColorSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.colorBottomGraphicBid); }
     		set { helper.colorBottomGraphicBid = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		
		[XmlIgnore()]
		[Description("Ask Volume Color for Bottom Graphic")]
		[Category("10. Bottom Graphic")]
		[Gui.Design.DisplayName("07.  Ask Volume Color")]
		
		public Color BGAskColor
		{
			get { return (helper.colorBottomGraphicAsk); }
			set {  helper.colorBottomGraphicAsk=value;}
		}
		
		[Browsable(false)]
		public string BGAskColorSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(helper.colorBottomGraphicAsk); }
     		set { helper.colorBottomGraphicAsk = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		  #endregion
		
		  #region BA Ladder
		[Description("Show or hide the BA ladders")]
		[Category("05. BA Ladder")]
		[Gui.Design.DisplayName("01. Show")]

		public bool ShowBALadder
		{
			get {return helper.showBALadder;}
			set {helper.showBALadder=value;}
		}
		
		[Description("")]
		[Category("05. BA Ladder")]
		[Gui.Design.DisplayName("02. Number Format")]
		
		public FormattingType BALadderFormat
		{
			get { return helper.ladderBAFormat; }
			set {  helper.ladderBAFormat=value;}
		}
		
		
		[Description("Ladder Display Type. BidAndAsk,VolAndDelta,DeltaColoredVolumeBidAsk,DeltaOnly")]
		[Category("05. BA Ladder")]
		[Gui.Design.DisplayName("03. Ladder Display Type")]

		public CenterLadderType Clt
		{
			get {return helper.clt;}
			set {helper.clt=value;}
		}
		
		[Description("Show ER Ladder")]
		[Category("05. BA Ladder")]
		[Gui.Design.DisplayName("04. Show ERladder")]

		public bool ShowERLadder
		{
			get {return helper.ShowERLadder;}
			set {helper.ShowERLadder=value;}
		}
		
		
		[Description("Used only if above is fixed number. Determines max value expected. Above values are displayed 100%")]
		[Category("05. BA Ladder")]
		[Gui.Design.DisplayName("05. Max Number")]

		public int BARatioNumber
		{
			get { return helper.ladderBAFixedNumber; }
			set {  helper.ladderBAFixedNumber=value;}
		}
				
		[Description("Show Maximum Volume Level : POC of bar")]
		[Category("05. BA Ladder")]
		[Gui.Design.DisplayName("06. Show Max Volume Level")]

		public bool ShowMaxLevel
		{
			get { return helper.ladderBAShowMaxLevel; }
			set {  helper.ladderBAShowMaxLevel=value;}
		}
		
		[Description("Show COT")]
		[Category("05. BA Ladder")]
		[Gui.Design.DisplayName("07. Show COT")]

		public bool ShowCOT
		{
			get { return helper.ladderBAShowCOT; }
			set {  helper.ladderBAShowCOT=value;}
		}
		
		[Description("Show HotKey")]                 // Added by  devdas
		[Category("05. BA Ladder")]                   //
		[Gui.Design.DisplayName("08. Show HotKey")]  //
		public bool Hotkeys                      //
		{
			get { return helper.showhotkeys; }
			set {  helper.showhotkeys=value;}
		}                                            //
		
		[Description("Center Ladder Cell Fill Type")]
		[Category("05. BA Ladder")]
		[Gui.Design.DisplayName("09. Center Fill Type")]

		public CellFillType  CftCenter
		{
			get {return helper.cftCenter;}
			set {helper.cftCenter=value;}
		}
		
		[Description("No minus sign")]
		[Category("05. BA Ladder")]
		[Gui.Design.DisplayName("10. No Minus Sign")]

		public bool NoMinus
		{
			get { return helper.ladderBANoMinus; }
			set {  helper.ladderBANoMinus=value;}
		}
		
		
		[Description("Show/Hide VWAP")]
		[Category("05. BA Ladder")]
		[Gui.Design.DisplayName("11. Show VWAP")]

		public bool ShowVWAP
		{
			get {return helper.showVWAP;}
			set {helper.showVWAP=value;}
		}
		
		[Description("Show Outer Border on High Side")]
		[Category("05. BA Ladder")]
		[Gui.Design.DisplayName("12. Show Outer Border")]

		public bool ShowOuterBorder
		{
			get { return helper.ladderBAShowOuterBoarder; }
			set {  helper.ladderBAShowOuterBoarder=value;}
		}
		
		[Description("Show only high side delta/vol bars in BA ladder")]
		[Category("05. BA Ladder")]
		[Gui.Design.DisplayName("13. High Side Bars")]

		public bool ShowBALadderHighSide
		{
			get {return helper.ladderBAHighSideMode;}
			set {helper.ladderBAHighSideMode=value;}
		}
		
		[Description("CutOff Percent for delta/vol bars in BA ladder in High Side Mode")]
		[Category("05. BA Ladder")]
		[Gui.Design.DisplayName("14. High Side Percent")]

		public int ShowBALadderHighSidePercent
		{
			get {return helper.ladderBAHighSidePercent;}
			set {helper.ladderBAHighSidePercent=Math.Min(Math.Max(1,value),100);}
		}
		
		[Description("Show BA bars widths in comparison to max volume either of bid/ask ladder")]
		[Category("05. BA Ladder")]
		[Gui.Design.DisplayName("15. BA True Shape")]

		public bool ShowBABarsTrueShape
		{
			get {return helper.ladderBATrueShape;}
			set {helper.ladderBATrueShape=value;}
		}
		
		[Description("Cell Color Scheme of Ladder")]
		[Category("05. BA Ladder")]
		[Gui.Design.DisplayName("16. Cell Color Scheme")]

		public CellColorScheme Ccs
		{
			get {return helper.ccs;}
			set {helper.ccs=value;}
		}
		
		
		[Description("Show BidxAsk  instead of Volume in DelatColredVolumeBidAsk Center Ladder Type")]
		[Category("05. BA Ladder")]
		[Gui.Design.DisplayName("17. BidAsk on DeltaBase")]

		public bool ShowBidAskOnDeltaBase
		{
			get {return helper.ladderBAShowBidAskOnDeltaBase;}
			set {helper.ladderBAShowBidAskOnDeltaBase=value;}
		}
		
		#endregion
		
		  #region LowerDataBoxes
		
		[Description("Select Bottom View Type None/Statistic/Graphic")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("01. Bottom View")]

		public BottomViewType Bottomview
		{
			get { return helper.bottomview; }
			set {  helper.bottomview=value;}
		}
		
		
		// Data Rows Settings 
		[Description("Select Row for Delta Display")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("02. Delta Row")]

		public RowNumber DeltaRow
		{
			get { return helper.deltaRow; }
			set {  helper.deltaRow=value;}
		}
		
		[Description("Select Row for Cummulative Delta Display")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("03. Cum Delta Row")]

		public RowNumber CumDeltaRow
		{
			get { return helper.cumDeltaRow; }
			set {  helper.cumDeltaRow=value;}
		}
		
		[Description("Select Row for Volume Display")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("04. Volume Row")]

		public RowNumber VolRow
		{
			get { return helper.volRow; }
			set {  helper.volRow=value;}
		}
		
		[Description("Select Row for Cummulative Volume Display")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("05. Cum Volume Row")]

		public RowNumber CumVolRow
		{
			get { return helper.cumVolRow; }
			set {  helper.cumVolRow=value;}
		}
		
		[Description("Select Row for Percent Cummulative Delta Display")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("06. Percent CumDelta Row")]

		public RowNumber PctCumDeltaRow
		{
			get { return helper.pctCumDeltaRow; }
			set {  helper.pctCumDeltaRow=value;}
		}
		
		[Description("Select Row for Seller Percent Display")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("07. Percent Seller Row")]

		public RowNumber PctSellerRow
		{
			get { return helper.pctSellerRow; }
			set {  helper.pctSellerRow=value;}
		}
		
		[Description("Select Row for Buyer Percent Display")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("08. Percent Buyer Row")]

		public RowNumber PctBuyerRow
		{
			get { return helper.pctBuyerRow; }
			set {  helper.pctBuyerRow=value;}
		}
		
		[Description("Select Row for Buyer Display")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("09. Buyer Row")]

		public RowNumber BuyerRow
		{
			get { return helper.buyerRow; }
			set {  helper.buyerRow=value;}
		}
		
		[Description("Select Row for Seller Display")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("10. Seller Row")]

		public RowNumber SellerRow
		{
			get { return helper.sellerRow; }
			set {  helper.sellerRow=value;}
		}
		
		[Description("Select Row for Percent MinDelta Display")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("11. Percent MinDelta Row")]

		public RowNumber PctMinDeltaRow
		{
			get { return helper.pctMinDeltaRow; }
			set {  helper.pctMinDeltaRow=value;}
		}
		
		[Description("Select Row for Percent MaxDelta Display")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("12. Percent MaxDelta Row")]

		public RowNumber PctMaxDeltaRow
		{
			get { return helper.pctMaxDeltaRow; }
			set {  helper.pctMaxDeltaRow=value;}
		}
		
		[Description("Select Row for Percent Delta Display")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("13. Percent Delta Row")]

		public RowNumber PctDeltaRow
		{
			get { return helper.pctDeltaRow; }
			set {  helper.pctDeltaRow=value;}
		}
		
		[Description("Select Row for MinDelta Display")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("14. MinDelta Row")]

		public RowNumber MinDeltaRow
		{
			get { return helper.minDeltaRow; }
			set {  helper.minDeltaRow=value;}
		}
		
		[Description("Select Row for MaxDelta Display")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("15. MaxDelta Row")]

		public RowNumber MaxDeltaRow
		{
			get { return helper.maxDeltaRow; }
			set {  helper.maxDeltaRow=value;}
		}
		
		
		[Description("Select Row for Finish Display")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("16. Finish Row")]

		public RowNumber FinishRow
		{
			get { return helper.finishRow; }
			set {  helper.finishRow=value;}
		}
		
		
		[Description("Select Row for Start Display")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("17. Start Row")]

		public RowNumber StartRow
		{
			get { return helper.startRow; }
			set {  helper.startRow=value;}
		}
		
		[Description("Show abbreviation in Bottom boxes.")]
		[Category("09. Bottom View")]
		[Gui.Design.DisplayName("18. Abbreviation")]

		public bool Abrev
		{
			get {return helper.abrev;}
			set {helper.abrev=value;}
		}
		
		#endregion
		
		  #region Fonts
		
		[XmlIgnore()]
		[Description("Select BidAsk Ladder Font")]
		[Category("07. Fonts")]
		[Gui.Design.DisplayName("01. BA Font")]
		public Font BAfont		
		{
			get { return helper.baFont; }
			set { helper.baFont = value; }
		}	
		
		[Browsable(false)]
		public string BAfontSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableFont.ToString(helper.baFont); }
     		set { helper.baFont = NinjaTrader.Gui.Design.SerializableFont.FromString(value); }
		}
		
		
		
		[XmlIgnore()]
		[Description("Select Side Ladder Font")]
		[Category("07. Fonts")]
		[Gui.Design.DisplayName("02. Side Ladder Font")]
		public Font Sidefont		
		{
			get { return helper.sideFont; }
			set { helper.sideFont = value; }
		}	
		
		[Browsable(false)]
		public string SidefontSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableFont.ToString(helper.sideFont); }
     		set { helper.sideFont = NinjaTrader.Gui.Design.SerializableFont.FromString(value); }
		}
		
		
		[XmlIgnore()]
		[Description("Select Bottom Boxes Font")]
		[Category("07. Fonts")]
		[Gui.Design.DisplayName("03. Bottom Boxes Font")]
		public Font Bottomfont		
		{
			get { return helper.bottomFont; }
			set { helper.bottomFont = value; }
		}	
		
		[Browsable(false)]
		public string BottomfontSerialize
		{
    		get { return NinjaTrader.Gui.Design.SerializableFont.ToString(helper.bottomFont); }
     		set { helper.bottomFont = NinjaTrader.Gui.Design.SerializableFont.FromString(value); }
		}
		
		#endregion
	
		  #region CR Setting
		[Description("Show Long Reversal")]
		[Category("08. Contract Reversal Settings")]
		[Gui.Design.DisplayName("1. Show Long Reversal")]

		public bool ShowUpReversal
		{
			get { return helper.ladderBAShowUpReversal; }
			set {  helper.ladderBAShowUpReversal=value;}
		}
		
			
		[Description("Show Short Reversal")]
		[Category("08. Contract Reversal Settings")]
		[Gui.Design.DisplayName("2. Show Short Reversal")]

		public bool ShowDwnReversal
		{
			get { return helper.ladderBAShowDwnReversal; }
			set {  helper.ladderBAShowDwnReversal=value;}
		}
		#endregion
		
		
		
		[Description("Sets the amount needed to display a contract reversal")]
		[Category("08. Contract Reversal Settings")]
		[Gui.Design.DisplayName("3. Contract Threshold")]

		public int ContractThreshold
		{
			get { return helper.contractReversalSetting; }
			set {  helper.contractReversalSetting=value;}
		}
		
		#region Numbers/Borders
		[Description("Show Numbers And Borders")]
		[Category("5a. Show Numbers and Borders")]
		[Gui.Design.DisplayName("01. Numbers, BA Ladder")]

		public bool ShowNumbers
		{
			get { return helper.ladderBAShowNumbers; }
			set {  helper.ladderBAShowNumbers=value;}
		}
		
		
		
		[Description("Shows or hides borders  on BA ladder")]
		[Category("5a. Show Numbers and Borders")]
		[Gui.Design.DisplayName("02. Borders, BA Ladders")]

		public bool ShowBordersBA
		{
			get {return helper.showBordersBA;}
			set {helper.showBordersBA=value;}
		}
	
		[Description("Shows or hides borders and numbers on Side Ladders")]
		[Category("5a. Show Numbers and Borders")]
		[Gui.Design.DisplayName("03. Numbers, Side Ladders")]

		public bool ShowNumbersSide
		{
			get {return helper.showNumbersSide;}
			set {helper.showNumbersSide=value;}
		}
		
		[Description("Shows or hides borders  on BA ladder")]
		[Category("5a. Show Numbers and Borders")]
		[Gui.Design.DisplayName("04. Borders, Side Ladders")]

		public bool ShowBordersSide
		{
			get {return helper.showBordersSide;}
			set {helper.showBordersSide=value;}
		}
		
		[Description("Shows/Hides/Locate Bottom Legend")]
		[Category("5a. Show Numbers and Borders")]
		[Gui.Design.DisplayName("05. Locate Legend")]

		public LegendLocation BottomLL
		{
			get {return helper.bottomLL;}
			set {helper.bottomLL=value;}
		}
		
		[Description("Extra Right Side Space after Side Ladders( 1-150")]
		[Category("5a. Show Numbers and Borders")]
		[Gui.Design.DisplayName("06. Extra Right Space")]

		public int ExtraRightSpace
		{
			get {return helper.extraRightSpace;}
			set {helper.extraRightSpace=Math.Min(150,Math.Max(1,value));}
		}
		#endregion
		
		
		[Browsable(false)]
		[XmlIgnore()]
		public IntSeries HighCOT
		{
			get 
			{ 
				return helper.dsHighCOT; 
			}
		}

		[Browsable(false)]
		[XmlIgnore()]
		public IntSeries LowCOT
		{
			get 
			{ 
				return helper.dsLowCOT; 
			}
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public IntSeries MaxBid
		{
			get 
			{ 
				return helper.dsMaxBid; 
			}
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public IntSeries MaxAsk
		{
			get 
			{ 
				return helper.dsMaxAsk; 
			}
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public IntSeries SumBid
		{
			get 
			{ 
				return helper.dsSumBid; 
			}
		}
				
		[Browsable(false)]
		[XmlIgnore()]
		public IntSeries SumAsk
		{
			get 
			{ 
				return helper.dsSumAsk; 
			}
		}
				
		[Browsable(false)]
		[XmlIgnore()]
		public IntSeries Delta
		{
			get 
			{ 
				return helper.dsDelta; 
			}
		}
				
		[Browsable(false)]
		[XmlIgnore()]
		public IntSeries Volume
		{
			get 
			{ 
				return helper.dsVolume; 
			}
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public IntSeries SessionCD
		{
			get 
			{ 
				return helper.dsSessionCD; 
			}
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public IntSeries SessionCV
		{
			get 
			{ 
				return helper.dsSessionCV; 
			}
		}
		
		
		[Browsable(false)]
		[XmlIgnore()]
		public IntSeries CumCD
		{
			get 
			{ 
				return helper.dsCumCD; 
			}
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public IntSeries SessionVWAP
		{
			get 
			{ 
				return helper.dsSessionVWAP; 
			}
		}
		
		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private VolumeLadderMetroEditionV2[] cacheVolumeLadderMetroEditionV2 = null;

        private static VolumeLadderMetroEditionV2 checkVolumeLadderMetroEditionV2 = new VolumeLadderMetroEditionV2();

        /// <summary>
        /// Shows the volumes of Buys and Sells at each price level within the bars of the price chart.
        /// </summary>
        /// <returns></returns>
        public VolumeLadderMetroEditionV2 VolumeLadderMetroEditionV2()
        {
            return VolumeLadderMetroEditionV2(Input);
        }

        /// <summary>
        /// Shows the volumes of Buys and Sells at each price level within the bars of the price chart.
        /// </summary>
        /// <returns></returns>
        public VolumeLadderMetroEditionV2 VolumeLadderMetroEditionV2(Data.IDataSeries input)
        {
            if (cacheVolumeLadderMetroEditionV2 != null)
                for (int idx = 0; idx < cacheVolumeLadderMetroEditionV2.Length; idx++)
                    if (cacheVolumeLadderMetroEditionV2[idx].EqualsInput(input))
                        return cacheVolumeLadderMetroEditionV2[idx];

            lock (checkVolumeLadderMetroEditionV2)
            {
                if (cacheVolumeLadderMetroEditionV2 != null)
                    for (int idx = 0; idx < cacheVolumeLadderMetroEditionV2.Length; idx++)
                        if (cacheVolumeLadderMetroEditionV2[idx].EqualsInput(input))
                            return cacheVolumeLadderMetroEditionV2[idx];

                VolumeLadderMetroEditionV2 indicator = new VolumeLadderMetroEditionV2();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                Indicators.Add(indicator);
                indicator.SetUp();

                VolumeLadderMetroEditionV2[] tmp = new VolumeLadderMetroEditionV2[cacheVolumeLadderMetroEditionV2 == null ? 1 : cacheVolumeLadderMetroEditionV2.Length + 1];
                if (cacheVolumeLadderMetroEditionV2 != null)
                    cacheVolumeLadderMetroEditionV2.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheVolumeLadderMetroEditionV2 = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Shows the volumes of Buys and Sells at each price level within the bars of the price chart.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.VolumeLadderMetroEditionV2 VolumeLadderMetroEditionV2()
        {
            return _indicator.VolumeLadderMetroEditionV2(Input);
        }

        /// <summary>
        /// Shows the volumes of Buys and Sells at each price level within the bars of the price chart.
        /// </summary>
        /// <returns></returns>
        public Indicator.VolumeLadderMetroEditionV2 VolumeLadderMetroEditionV2(Data.IDataSeries input)
        {
            return _indicator.VolumeLadderMetroEditionV2(input);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Shows the volumes of Buys and Sells at each price level within the bars of the price chart.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.VolumeLadderMetroEditionV2 VolumeLadderMetroEditionV2()
        {
            return _indicator.VolumeLadderMetroEditionV2(Input);
        }

        /// <summary>
        /// Shows the volumes of Buys and Sells at each price level within the bars of the price chart.
        /// </summary>
        /// <returns></returns>
        public Indicator.VolumeLadderMetroEditionV2 VolumeLadderMetroEditionV2(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.VolumeLadderMetroEditionV2(input);
        }
    }
}
#endregion
